#ifndef __Mesh__
#define __Mesh__


/**
 * a Mesh
 */
class Mesh
{
	public:
		LPDIRECT3DDEVICE9	d3ddev;
		LPD3DXMESH			mesh;
		D3DMATERIAL9*		material;
		LPDIRECT3DTEXTURE9*	texture;
		DWORD				numMaterials;	

		Mesh();
		~Mesh();

		void		draw	 ( );
		static D3DXMATRIX	translate( float, float, float );
		static D3DXMATRIX	rotate	 ( float, float, float );
		static D3DXMATRIX	scale	 ( float, float, float );
};

Mesh::Mesh()
{
	DirectXManager  & dx	= DirectXManager	::getInstance();
	d3ddev					= dx.d3ddev;
	mesh					= NULL;
}

Mesh::~Mesh()
{
	if( mesh != NULL )
		mesh->Release();
}


/**
 * Draw a mesh
 * @param name name of the mesh
 */
void Mesh::draw()
{
    // draw the spaceship3
	for(DWORD i = 0; i < numMaterials; i++)
    {
        d3ddev->SetMaterial(&material[i]);

        if(texture[i] != NULL)
		{
			d3ddev->SetTexture(  0 , texture[i]);
			d3ddev->SetSamplerState(0, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR);
		}


        mesh->DrawSubset(i);
    }
}


/**
 * Rotate the object by the all axes
 * @parap x x
 * @parap y y
 * @parap z z
 */
D3DXMATRIX Mesh::translate( float x, float y, float z )
{
	D3DXMATRIX matTrans;
	D3DXMatrixTranslation(&matTrans, x, y, z);
	return matTrans;
}


/**
 * Rotate the object by the all axes
 * @parap x x
 * @parap y y
 * @parap z z
 */
D3DXMATRIX Mesh::rotate( float x, float y, float z )
{
	D3DXMATRIX matRotateX;
	D3DXMATRIX matRotateY;
	D3DXMATRIX matRotateZ;

	D3DXMatrixRotationX(&matRotateX, D3DXToRadian(x));
	D3DXMatrixRotationY(&matRotateY, D3DXToRadian(y));
	D3DXMatrixRotationZ(&matRotateZ, D3DXToRadian(z));

	matRotateX =  matRotateY * matRotateX * matRotateZ;

	return matRotateX;
}


/**
 * Scale the object by the all axes
 * @parap x x
 * @parap y y
 * @parap z z
 */
D3DXMATRIX Mesh::scale( float x, float y, float z )
{
	D3DXMATRIX mat;
	D3DXMatrixScaling(&mat,x,y,z);
	return mat;
}


#endif