#ifndef __NOTIFICATION__
#define __NOTIFICATION__

#include <iostream>

#include "../Managers/SpritesManager.h"
#include "../Elements/Object2D.h"

#define NOTIFICATION_TIME	1000

/**
 * Menu
 *
 * BAUDOUX Jrme - TIXIER Benot
 */
class Notification : public Object2D
{
	protected:
		SpritesManager  * sp;

		long			notificationTime;
		long			timeNotificationRemaining;

	public:
		Notification(int _x,int _y,int _width,int _height);
		~Notification();

		inline  void setTime( long );
		inline  bool isFinished();
		virtual void refresh( long );
		virtual void draw();

	protected:
};

/**
 * Constructor
 */
Notification::Notification(int _x, int _y, int _width, int _height) : Object2D(_x,_y,0,_width,_height)
{
	notificationTime			= NOTIFICATION_TIME;
	sp							= &SpritesManager ::getInstance();
	timeNotificationRemaining	= notificationTime;
}

/**
 * Destructor
 */
Notification::~Notification()
{

}

/**
 * Set the time of the notification
 */
void Notification::setTime( long time )
{
	notificationTime			= time;
	timeNotificationRemaining	= time;
}

/**
 * If the notification is over
 */
bool Notification::isFinished()
{
	return timeNotificationRemaining <= 0;
}


/**
 * Refresh the menu
 */
void Notification::refresh( long time )
{
	Object2D::refresh(time);
	timeNotificationRemaining -= time;
}


/**
 * Draw the notification
 */
void Notification::draw()
{
	// INIT
	int a,r,g,b;
	a = r = g = b = 255;

	if( timeNotificationRemaining > notificationTime*3/4.0 )
	{
		a = 255 - (int)((( timeNotificationRemaining - notificationTime*3/4.0 ) / (notificationTime/4.0) ) * 255);
	}
	else if( timeNotificationRemaining < notificationTime/4.0 )
	{
		a = (int)(( timeNotificationRemaining / (notificationTime/4.0) ) * 255);
	}

	if( sprite != NULL )
	{
		// RESIZE
		if( width != 0 && height != 0 )
		{
			D3DXVECTOR2 spriteCentre=D3DXVECTOR2(0,0);
			D3DXVECTOR2 trans=D3DXVECTOR2((float)x,(float)y);
			D3DXMATRIX mat;
			D3DXVECTOR2 scaling( (float)width/1024.0f, (float)height/1024.0f );
			D3DXMatrixTransformation2D(&mat,NULL,0.0,&scaling,&spriteCentre,0,&trans);
			d3dspt->SetTransform(&mat);
			d3dspt->Draw( sprite, NULL, NULL, NULL, D3DCOLOR_ARGB(a, r, g, b));
		}
		else
		{
			D3DXVECTOR3 position((float)x, (float)y, (float)z);
			d3dspt->Draw( sprite, NULL, NULL, &position, D3DCOLOR_ARGB(a, r, g, b));
		}
	}
}

#endif
