#ifndef __2DObject__
#define __2DObject__

#include "Point2D.h"
#include "../Managers/SpritesManager.h"

class Object2D : public Point2D
{
	protected:
		double				width;
		double				height;
		LPDIRECT3DTEXTURE9	sprite;
		LPD3DXSPRITE		d3dspt;

	public:

		Object2D();
		Object2D( double _x, double _y, double _z );
		Object2D( double _x, double _y, double _z, double _width, double _height );
		Object2D( double _x, double _y, double _z, double _width, double _height, double _ry, double _vx, double _vy );

		inline double getWidth() const;
		inline double getHeight() const;
		inline LPDIRECT3DTEXTURE9 getSprite();
		inline void setWidth(double);
		inline void setHeight(double);
		inline void setSprite( LPDIRECT3DTEXTURE9 );

		virtual void draw();
		virtual void refresh( long temp );
};

Object2D::Object2D() : Point2D()
{
	SpritesManager & sp = SpritesManager::getInstance();
	d3dspt				= sp.getSpriteDevice();

	width	= 0;
	height	= 0;

	sprite = NULL;
}

Object2D::Object2D( double _x, double _y, double _z ) : Point2D(_x,_y,_z)
{
	SpritesManager & sp = SpritesManager::getInstance();
	d3dspt				= sp.getSpriteDevice();

	width	= 0;
	height	= 0;

	sprite = NULL;
}

Object2D::Object2D( double _x, double _y, double _z, double _width, double _height ) : Point2D(_x,_y,_z)
{
	SpritesManager & sp = SpritesManager::getInstance();
	d3dspt				= sp.getSpriteDevice();

	width  = _width;
	height = _height;

	sprite = NULL;
}

Object2D::Object2D( double _x, double _y, double _z, double _width, double _height, double _ry, double _vx, double _vy ) : Point2D(_x,_y,_z,_ry,_vx,_vy)
{
	SpritesManager & sp = SpritesManager::getInstance();
	d3dspt				= sp.getSpriteDevice();

	width  = _width;
	height = _height;

	sprite = NULL;
}

double Object2D::getWidth() const
{
	return width;
}

double Object2D::getHeight() const
{
	return height;
}

LPDIRECT3DTEXTURE9 Object2D::getSprite()
{
	return sprite;
}

void Object2D::setWidth( double _width )
{
	width = _width;
}

void Object2D::setHeight( double _height )
{
	height = _height;
}

void Object2D::setSprite( LPDIRECT3DTEXTURE9 sp )
{
	sprite = sp;
}

void Object2D::refresh( long temp )
{
	Point2D::refresh( temp );
}



void Object2D::draw()
{
	if( sprite != NULL )
	{
		// RESIZE
		if( width != 0 && height != 0 )
		{
			D3DXVECTOR2 spriteCentre=D3DXVECTOR2(0,0);
			D3DXVECTOR2 trans=D3DXVECTOR2((float)x,(float)y);
			D3DXMATRIX mat;
			D3DXVECTOR2 scaling( (float)width/1024.0f, (float)height/1024.0f );
			D3DXMatrixTransformation2D(&mat,NULL,0.0,&scaling,&spriteCentre,0,&trans);
			d3dspt->SetTransform(&mat);
			d3dspt->Draw( sprite, NULL, NULL, NULL, 0xFFFFFFFF);
		}
		else
		{
			D3DXVECTOR3 position((float)x, (float)y, (float)z);
			d3dspt->Draw( sprite, NULL, NULL, &position, 0xFFFFFFFF);
		}
	}
}


#endif