#ifndef __3DObject__
#define __3DObject__

#include "Point3D.h"
#include "../Managers/MeshesManager.h"

class Object3D : public Point3D
{
	protected:
		double	sx;
		double	sy;
		double	sz;

		Mesh *	mesh;
		LPDIRECT3DDEVICE9 d3ddev;

	public:

		Object3D();
		Object3D( double _x, double _y, double _z );
		Object3D( double _x, double _y, double _z, double _sx, double _sy, double _sz );
		Object3D( double _x, double _y, double _z, double _sx, double _sy, double _sz, double _rx, double _ry, double _rz, double _vx, double _vy, double _vz );

		inline double getSX() const;
		inline double getSY() const;
		inline double getSZ() const;
		inline Mesh*  getMesh();

		inline void setSX(double);
		inline void setSY(double);
		inline void setSZ(double);
		inline void setMesh( Mesh* );

		virtual void draw();
		virtual void refresh( long temp );
};

Object3D::Object3D() : Point3D()
{
	DirectXManager  & dx	= DirectXManager	::getInstance();
	d3ddev					= dx.d3ddev;

	sx = 1;
	sy = 1;
	sz = 1;

	mesh = NULL;
}

Object3D::Object3D( double _x, double _y, double _z ) : Point3D(_x,_y,_z)
{
	DirectXManager  & dx	= DirectXManager	::getInstance();
	d3ddev					= dx.d3ddev;

	sx = 1;
	sy = 1;
	sz = 1;

	mesh = NULL;
}

Object3D::Object3D( double _x, double _y, double _z, double _sx, double _sy, double _sz ) : Point3D(_x,_y,_z)
{
	DirectXManager  & dx	= DirectXManager	::getInstance();
	d3ddev					= dx.d3ddev;

	sx = _sx;
	sy = _sy;
	sz = _sz;

	mesh = NULL;
}

Object3D::Object3D( double _x, double _y, double _z, double _sx, double _sy, double _sz, double _rx, double _ry, double _rz, double _vx, double _vy, double _vz ) : Point3D(_x,_y,_z,_rx,_ry,_rz,_vx,_vy,_vz)
{
	DirectXManager  & dx	= DirectXManager	::getInstance();
	d3ddev					= dx.d3ddev;

	sx = _sx;
	sy = _sy;
	sz = _sz;

	mesh = NULL;
}

double Object3D::getSX() const
{
	return sx;
}

double Object3D::getSY() const
{
	return sy;
}

double Object3D::getSZ() const
{
	return sz;
}

Mesh * Object3D::getMesh()
{
	return mesh;
}



void Object3D::setSX( double _sx )
{
	sx = _sx;
}

void Object3D::setSY( double _sy )
{
	sy = _sy;
}

void Object3D::setSZ( double _sz )
{
	sz = _sz;
}



void Object3D::setMesh( Mesh * _mesh )
{
	mesh = _mesh;
}


void Object3D::refresh( long temp )
{
	Point3D::refresh( temp );
}



void Object3D::draw()
{
	if( mesh != NULL )
	{
		D3DXMATRIX matTrans;
		matTrans = Mesh::rotate((float)rx,(float)ry,(float)rz) * Mesh::scale((float)sx,(float)sy,(float)sz) * Mesh::translate((float)x,(float)y,(float)z) ;
		d3ddev->SetTransform(D3DTS_WORLD, &(matTrans));
		mesh->draw();
	}
}


#endif