#ifndef __Robot__
#define __Robot__

#include <cmath>
#include "Object3D.h"
#include "../Managers/MeshesManager.h"
#include "../Managers/SoundsManager.h"
#include "../Managers/ControlsManager.h"
#include "../Elements/Frisbee.h"

#define ROBOT_MODE_NORMAL				0
#define ROBOT_MODE_BOOST				1
#define ROBOT_MODE_GOTFRISBEE			2

#define ROBOT_VITESSE_DIAGONALE			1.2
#define ROBOT_VITESSE_AVANCE			1.6
#define ROBOT_BOOST_IMPULSION			3.0
#define ROBOT_BOOST_DECELERATION		4.0
#define ROBOT_TIME_KEEP_FRISBEE			4000
#define ROBOT_SPEED_LAUNCH_POWER		1.25
#define ROBOT_SPEED_LAUNCH_MIN			1
#define ROBOT_SPEED_LAUNCH_LOBE_POWER	.4
#define ROBOT_SPEED_LAUNCH_LOBE_MIN		2

#define ROBOT_SPEED_ROTATION			.4

class Robot : public Object3D
{
	protected:

		// RESSOURSES
		Mesh *	head;
		Mesh *	body;

		Sound * throwSound;
		Sound * moveSoud;
		Sound * strafSound;
		Sound * getFrisbeeSound;

		ControlsManager   * ctrl;

		// VALUES
		double	lookAtX;
		double	lookAtY;
		double	angleLookAt;

		double  sizeX;
		double  sizeY;
		double  sizeZ;

		int		mode;
		int		player;
		int     type;

		float	power;
		float	speed;

		double	flip;

		double  timeRemainingGotFrisbee;

		double	frisbeeSpeedX;
		double	frisbeeSpeedY;
		double	frisbeeSpeedZ;

		double	angleExpectedY;

	public:

		Robot( double _x, double _y, int _player, int _type);

		inline  void replace( int _x, int _y );
		virtual void draw();
		virtual void refresh( long temp );
		virtual void collision( Frisbee * _frisbee );
};

/**
 * Constructor
 */
Robot::Robot( double _x, double _y, int _player, int _type ) : Object3D( _x,_y,-5,.3,.3,.3)
{
	// MANAGERS
	ctrl = &ControlsManager::getInstance();

	std::string headstr;
	std::string bodystr;
	std::string meshstr;
	std::string headname;
	std::string bodyname;
	std::string meshname;

	switch( _type )
	{
		case 0:
			headstr  = std::string( "Meshes/Robot/Small/head.x" );
			bodystr  = std::string( "Meshes/Robot/Small/body.x" );
			meshstr  = std::string( "Meshes/Robot/Small/bottom.x" );
			headname = std::string( "SmallHead" );
			bodyname = std::string( "SmallBody" );
			meshname = std::string( "SmallBottom" );

			power = 1;
			speed = 1.35f;

			sizeX = .5;
			sizeY = .5;
			sizeZ =  2;

			break;

		case 1:
			headstr  = std::string( "Meshes/Robot/Big/head.x" );
			bodystr  = std::string( "Meshes/Robot/Big/body.x" );
			meshstr  = std::string( "Meshes/Robot/Big/bottom.x" );
			headname = std::string( "BigHead" );
			bodyname = std::string( "BigBody" );
			meshname = std::string( "BigBottom" );

			power = 1.2f;
			speed = 1;

			sizeX = .7;
			sizeY = .7;
			sizeZ =  2.2;

			break;
	}

	// MESHES
	MeshesManager   & m = MeshesManager	::getInstance();
	head				= m.addMesh( headname, headstr );
	body				= m.addMesh( bodyname, bodystr ); 
	mesh				= m.addMesh( meshname, meshstr );

	// SOUNDS
	SoundsManager & snd = SoundsManager::getInstance();
	throwSound			= snd.addSound("throw"		,"Sounds/throw.wav"		,2);
	moveSoud			= snd.addSound("move"		,"Sounds/move.wav"		,2);
	strafSound			= snd.addSound("straf"		,"Sounds/straf.wav"		,2);
	getFrisbeeSound		= snd.addSound("getFrisbee"	,"Sounds/getFrisbee.wav",1);

	moveSoud		->setVolume(.7f);
	getFrisbeeSound	->setVolume(.6f);


	// VALUES
	mode					= ROBOT_MODE_NORMAL;
	player					= _player;
	type					= _type;

	angleLookAt				= 0;
	angleExpectedY			= 0;
	lookAtX					= 0;
	lookAtY					= 0;

	frisbeeSpeedX			= 0;
	frisbeeSpeedY			= 0;
	frisbeeSpeedZ			= 0;

	timeRemainingGotFrisbee	= 0;

	if( player == 1 )
		flip = 180;
	else
		flip = 0;
}

/**
 * Replace the robots
 */
void Robot::replace(int _x, int _y)
{
	mode					= ROBOT_MODE_NORMAL;

	angleLookAt				= 0;
	angleExpectedY			= 0;
	lookAtX					= 0;
	lookAtY					= 0;

	frisbeeSpeedX			= 0;
	frisbeeSpeedY			= 0;
	frisbeeSpeedZ			= 0;

	timeRemainingGotFrisbee	= 0;

	x = _x;
	y = _y;
}

/**
 * Refresh The palyers values
 * @param temp time between two frames
 */
void Robot::refresh( long temp )
{
	// IF CAN MOVE
	if( mode == ROBOT_MODE_NORMAL )
	{
		// MOUVE THE FOOTS 
		if( ctrl->isLeft(player) || ctrl->isRight(player) )
		{
			if( ctrl->isUp(player) )
			{
				vy = ROBOT_VITESSE_DIAGONALE * speed;

				if( ctrl->isLeft(player) )
					angleExpectedY = 45;
				else
					angleExpectedY = -45;
			}
			else if( ctrl->isDown(player) )
			{
				vy = - ROBOT_VITESSE_DIAGONALE * speed;

				if( ctrl->isLeft(player) )
					angleExpectedY = -45;
				else
					angleExpectedY = 45;
			}
			else
			{
				vy = 0;
				angleExpectedY = 0;
			}
		}
		else
		{
			if( ctrl->isUp(player) )
			{
				vy = ROBOT_VITESSE_AVANCE * speed;

				if( player == 2 )
					angleExpectedY = 90;
				else
					angleExpectedY = -90;
			}
			else if( ctrl->isDown(player) )
			{
				vy = - ROBOT_VITESSE_AVANCE * speed;

				if( player == 2 )
					angleExpectedY = -90;
				else
					angleExpectedY = 90;
			}
			else
			{
				vy = 0;

				angleExpectedY = 0;
			}
		}

		if( ctrl->isUp(player) || ctrl->isDown(player) )
		{
			if( ctrl->isLeft(player) )
			{
				vx = ROBOT_VITESSE_DIAGONALE * speed;
			}
			else if( ctrl->isRight(player) )
			{
				vx = - ROBOT_VITESSE_DIAGONALE * speed;
			}
			else
			{
				vx = 0;
			}
		}
		else
		{
			if( ctrl->isLeft(player) )
			{
				vx = ROBOT_VITESSE_AVANCE * speed;
			}
			else if( ctrl->isRight(player) )
			{
				vx = - ROBOT_VITESSE_AVANCE * speed;
			}
			else
			{
				vx = 0;
			}
		}

		if( ctrl->isB1(player) )
		{
			if( vx != 0 || vy != 0 )
			{
				mode = ROBOT_MODE_BOOST;
				vx*= ROBOT_BOOST_IMPULSION / std::sqrt(speed);
				vy*= ROBOT_BOOST_IMPULSION / std::sqrt(speed);

				strafSound->play();
			}
		}

		if( ctrl->isB2(player) )
		{

		}
	}
	// IF MODE BOOST OR THE ROBOT JUST TAKE THE FRISBEE
	else if( mode == ROBOT_MODE_BOOST || mode == ROBOT_MODE_GOTFRISBEE )
	{
		if( vx > -.5 && vx < .5 )
			vx = 0;
		else
			if( vx > 0 )
				vx -= ROBOT_BOOST_DECELERATION * temp / 500.0;
			else
				vx += ROBOT_BOOST_DECELERATION * temp / 500.0;

		if( vy > -.5 && vy < .5 )
			vy = 0;
		else
			if( vy > 0 )
				vy -= ROBOT_BOOST_DECELERATION * temp / 500.0;
			else
				vy += ROBOT_BOOST_DECELERATION * temp / 500.0;

		// IF THERE IS NO SPEED
		if( vx == 0 && vy == 0 && mode == ROBOT_MODE_BOOST )
		{
			mode = ROBOT_MODE_NORMAL;
			ctrl->initB1(player);
		}

		// IF THE PLAYER HAS THE FRISBEE
		if( mode == ROBOT_MODE_GOTFRISBEE )
		{

			// IF THE PLAYER LAUNCH THE FREESBE
			if( (ctrl->isB1(player) || ctrl->isB2(player) || timeRemainingGotFrisbee <= 0) && ROBOT_TIME_KEEP_FRISBEE - 300 > timeRemainingGotFrisbee  )
			{
				double angle;

				// CALCULATE THE ANGLE
				if( player == 1 )
				{
					angle = 180;

					if( ctrl->isUp(player) )
					{
						if( ctrl->isLeft(player) )
						{
							angle = 120;
						}
						else if( ctrl->isRight(player) )
						{
							angle = 160;
						}
						else
						{
							angle = 140;
						}
					}
					else if( ctrl->isDown(player) )
					{
						if( ctrl->isLeft(player) )
						{
							angle = -120;
						}
						else if( ctrl->isRight(player) )
						{
							angle = -160;
						}
						else
						{
							angle = -140;
						}
					}
				}
				else
				{
					angle = 0;

					if( ctrl->isUp(player) )
					{
						if( ctrl->isLeft(player) )
						{
							angle = 20;
						}
						else if( ctrl->isRight(player) )
						{
							angle = 60;
						}
						else
						{
							angle = 40;
						}
					}
					else if( ctrl->isDown(player) )
					{
						if( ctrl->isLeft(player) )
						{
							angle = -20;
						}
						else if( ctrl->isRight(player) )
						{
							angle = -60;
						}
						else
						{
							angle = -40;
						}
					}
				}


				frisbeeSpeedX = ( timeRemainingGotFrisbee/1000.0 + ROBOT_SPEED_LAUNCH_MIN ) * ROBOT_SPEED_LAUNCH_POWER * std::cos( D3DXToRadian(angle) ) * power;
				frisbeeSpeedY = ( timeRemainingGotFrisbee/1000.0 + ROBOT_SPEED_LAUNCH_MIN ) * ROBOT_SPEED_LAUNCH_POWER * std::sin( D3DXToRadian(angle) ) * power;

				// PLAY THE SOUND
				throwSound->play( -x/6.0, y/4.0, 0);

				// INITIATE THE BUTTON 1
				ctrl->initB1(player);
			}


			// IF THE PLAYER LAUNCH THE FREESBE BY A LOBE
			if( ctrl->isB2(player) && ROBOT_TIME_KEEP_FRISBEE - 300 > timeRemainingGotFrisbee  )
			{
				double angle;

				if( player == 1 )
				{
					angle = 180;

					if( ctrl->isUp(player) )
					{
						angle = 165;
					}
					else if( ctrl->isDown(player) )
					{
						angle = 195;	
					}
				}
				else
				{
					angle = 0;

					if( ctrl->isUp(player) )
					{
						angle = 15;
					}
					else if( ctrl->isDown(player) )
					{
						angle = -15;	
					}
				}


				frisbeeSpeedZ = 10;
				frisbeeSpeedX = ( timeRemainingGotFrisbee/2000.0 + ROBOT_SPEED_LAUNCH_LOBE_MIN ) * std::cos( D3DXToRadian(angle) ) * (( std::sqrt(x*x) + 1 )/7);
				frisbeeSpeedY = ( timeRemainingGotFrisbee/2000.0 + ROBOT_SPEED_LAUNCH_LOBE_MIN ) * std::sin( D3DXToRadian(angle) ) * (( std::sqrt(x*x) + 1 )/7);
				ctrl->initB2(player);
			}

			// SET THE TIME
			timeRemainingGotFrisbee -= temp;
			if( timeRemainingGotFrisbee < 0 )
				timeRemainingGotFrisbee = 0;
		}
	}

	

	//
	// FOOT DEPLACEMENT
	//

	// IF THE ANGLE EXPECTED IS NOT IN -180 180
	if( angleExpectedY > 180 )
		angleExpectedY -= 180;

	if( angleExpectedY < -180 )
		angleExpectedY += 180;

	// IF THE ANGLE IS NOT IN -180 180
	if( ry > 180 )
		ry -= 180;

	if( ry < -180 )
		ry += 180;

	// IF THE ANGLE IS NEAR THE EXPECTED ANGLE
	if( ry - angleExpectedY < 10 && ry - angleExpectedY > -10 )
		ry = angleExpectedY;
	// OTHERWISE
	else if( ry > angleExpectedY )
		ry -= ROBOT_SPEED_ROTATION * temp;
	else
		ry += ROBOT_SPEED_ROTATION * temp;


	//
	// THE HEAD LOOK AT ...
	//

	double a	= std::sqrt( (lookAtX - x) * (lookAtX - x) );
	double b	= std::sqrt( (lookAtY - y) * (lookAtY - y) );
	double h	= std::sqrt( a*a + b*b );
	angleLookAt = D3DXToDegree( std::acos( a/h ) );

	if( lookAtY - y < 0 )
		angleLookAt = - angleLookAt;

	if( lookAtX - x < 0 )
		angleLookAt = 180 - angleLookAt;

	Object3D::refresh( temp );


	// COLLISIONS WITH THE LEVEL
	if( y > 2.25 )
	{
		y	= 2.249;
		vx	= 0;
		vy  = 0;
	}

	if( y <-2.25 )
	{
		y   = -2.249;
		vx	= 0;
		vy  = 0;
	}

	if( x > 5.9 )
	{
		x   = 5.89;
		vx	= 0;
		vy  = 0;
	}

	if( x < -6 )
	{
		x   = -5.99;
		vx	= 0;
		vy  = 0;
	}

	if( player == 2 && x > -1  )
	{
		x   = -1.01;
		vx	= 0;
		vy  = 0;
	}

	if( player == 1 && x < 1 )
	{
		x   = 1.01;
		vx	= 0;
		vy  = 0;
	}

	// MOVMENT SOUNDS
	if( moveSoud->isPlayed( player-1 ) )
	{
		if( vx == 0 && vy == 0 )
		{
			moveSoud->stopSource( player-1 );
		}
		else
		{
			moveSoud->setSpeedSource( player-1, (vx*vx+vy*vy)/3.0 );
			moveSoud->setPositionSource( player-1, -x/6.0, y/4.0, 0 );
		}
	}
	else
	{
		if( vx !=0 || vy != 0 )
		{
			moveSoud->setSpeedSource( player-1, (vx*vx+vy*vy)/3.0 );
			moveSoud->playSource( player-1, -x/6.0, y/4.0, 0 );
		}
	}
}

/**
 * Draw the robot
 */
void Robot::draw()
{
	if( head != NULL )
	{
		D3DXMATRIX matTrans;
		matTrans = Mesh::rotate(90,(float)angleLookAt,90) * Mesh::scale((float)sx,(float)sy,(float)sz) * Mesh::translate((float)x,(float)y,(float)z) ;
		d3ddev->SetTransform(D3DTS_WORLD, &(matTrans));
		head->draw();
	}

	if( body != NULL )
	{
		D3DXMATRIX matTrans;
		matTrans = Mesh::rotate(90,(float)flip,90) * Mesh::scale((float)sx,(float)sy,(float)sz) * Mesh::translate((float)x,(float)y,(float)z) ;
		d3ddev->SetTransform(D3DTS_WORLD, &(matTrans));
		body->draw();
	}

	if( mesh != NULL )
	{
		D3DXMATRIX matTrans;
		matTrans = Mesh::rotate(90,(float)flip + (float)ry,90) * Mesh::scale((float)sx,(float)sy,(float)sz) * Mesh::translate((float)x,(float)y,(float)z) ;
		d3ddev->SetTransform(D3DTS_WORLD, &(matTrans));
		mesh->draw();
	}
}


/**
 * Collisions betwen the player and the frisbee
 */ 
void Robot::collision( Frisbee * _frisbee )
{
	// MODIFY THE ORIENTATION OF THE HEAD
	lookAtX = _frisbee->getX();
	lookAtY = _frisbee->getY();

	// IF THE ROBOT "WALK" STAY STILL OR MOVE BY BOOST
	if( mode == ROBOT_MODE_NORMAL || mode == ROBOT_MODE_BOOST )
	{
		// IF THE FRISBEE IS INSIDE THE ROBIT
		if( _frisbee->getX() > x - sizeX && _frisbee->getX() < x + sizeX && _frisbee->getY() > y - sizeY && _frisbee->getY() < y + sizeY &&  _frisbee->getZ() > -5.5 && _frisbee->getZ() < -5.5 + sizeZ && _frisbee->getLauncher() != player  && ( _frisbee->getMode() != FRISBEE_MODE_GHOST && _frisbee->getMode() != FRISBEE_MODE_LOST ) )
		{
			// TAKE THE FRISBEE
			_frisbee->init();
			_frisbee->setMode( FRISBEE_MODE_GHOST );
			_frisbee->setLauncher( player );
			mode					= ROBOT_MODE_GOTFRISBEE;
			timeRemainingGotFrisbee = ROBOT_TIME_KEEP_FRISBEE;

			getFrisbeeSound->play();
		}
	}
	// IF THE ROBOT HAS THE FRISBEE
	else if( mode == ROBOT_MODE_GOTFRISBEE )
	{
		// PUT THE FRISBEE JUST AFTER THE ROBOT
		if( player == 1 )
			_frisbee->setX( x - .1 );
		else
			_frisbee->setX( x + .1 );
		_frisbee->setY( y );	

		// IF WANT TO THROW THE FRISBEE ( MEANS THAT X IS NOT NULL )
		if( frisbeeSpeedX != 0 )
		{
			// TROW THE FRISBEE WITH THE GOOD SPEED
			_frisbee->setVX  ( frisbeeSpeedX );
			_frisbee->setVY  ( frisbeeSpeedY );
			_frisbee->setZ   ( FRISBEE_DEPTH_NORMAL );

			mode			= ROBOT_MODE_NORMAL;

			// IF LOBE ... OR COUP DROIT
			if( frisbeeSpeedZ != 0 )
			{
				_frisbee->setMode( FRISBEE_MODE_LOBE );
				_frisbee->setVZ  ( frisbeeSpeedZ );
			}
			else
			{
				_frisbee->setMode( FRISBEE_MODE_COUP_DROIT );
			}

			frisbeeSpeedX	= 0;
			frisbeeSpeedY	= 0;
			frisbeeSpeedZ	= 0;
		}
	}
}


#endif