#ifndef __ControlsManager__
#define __ControlsManager__

#include "Singleton.h"
#include "KeyboardManager.h"
#include "JoypadManager.h"

#include <dinput.h>
#include <iostream>
#include <fstream>
#include <sstream>

#define CONTROLMANAGER_NB_PLAYERS	2

/**
 * KeyboardManager
 *
 * BAUDOUX Jrme - TIXIER Benit
 */
class ControlsManager : public Singleton<ControlsManager>
{
	private:
		friend class Singleton<ControlsManager>; 

		int exit;

		int up		[CONTROLMANAGER_NB_PLAYERS];
		int down	[CONTROLMANAGER_NB_PLAYERS];
		int left	[CONTROLMANAGER_NB_PLAYERS];
		int right	[CONTROLMANAGER_NB_PLAYERS];
		int b1		[CONTROLMANAGER_NB_PLAYERS];
		int b2		[CONTROLMANAGER_NB_PLAYERS];

		KeyboardManager * key;
		JoypadManager   * joy;

	public:
		void refresh();

		inline  bool isEscape(  );
		inline  bool isEnter (  );

		inline  bool isUp	( int );
		inline  bool isDown	( int );
		inline  bool isLeft	( int );
		inline  bool isRight( int );
		inline  bool isB1	( int );
		inline  bool isB2	( int );

		inline  void init();
		inline  void init	( int );
		inline  void initAll	 ();
		inline  void initEscape  ();
		inline  void initEnter   ();
		inline  void initB1	( int );
		inline  void initB2 ( int );

		void clean();


		~ControlsManager();

	private:
		ControlsManager();


};


/**
 * Private constructor
 */
ControlsManager::ControlsManager()
{
	key = &KeyboardManager::getInstance();
	joy = &JoypadManager  ::getInstance();
}

/**
 * Public Destructor
 */
ControlsManager::~ControlsManager()
{
	clean();
}


/**
 * Initiate the keyboard
 */
void ControlsManager::init()
{
	key->init();
	joy->init();
    return; 
}

/**
 * Init all the keys
 */
void ControlsManager::initAll()
{
	for( int i=0; i< CONTROLMANAGER_NB_PLAYERS; i++)
		key->init(i);
}

/**
 * Init the keys of the player
 */
void ControlsManager::init( int player )
{
	up		[ (unsigned int) player-1 ]	= 0;
	down	[ (unsigned int) player-1 ]	= 0;
	left	[ (unsigned int) player-1 ]	= 0;
	right	[ (unsigned int) player-1 ]	= 0;
	b1		[ (unsigned int) player-1 ]	= 0;
	b2		[ (unsigned int) player-1 ]	= 0;

	key->init( player );
}

/**
 * Init the Esc button
 */
void ControlsManager::initEscape()
{
	exit = 3;
	key->initEscape();
}

/**
 * Init the Enter button
 */
void ControlsManager::initEnter()
{
	key->initEnter();
}

/**
 * Init the Ok button
 */
void ControlsManager::initB1( int player )
{
	b1		[ (unsigned int) player-1 ]	= 3;

	key->initB1( player );
}

/**
 * Init the No button
 */
void ControlsManager::initB2( int player )
{
	b2		[ (unsigned int) player-1 ]	= 3;	

	key->initB2( player );
}

/**
 * Check the keys press
 */
void ControlsManager::refresh()
{
	key->key();
	joy->refresh();

	for( int player=0; player < CONTROLMANAGER_NB_PLAYERS; player++ )
	{
		// UP
		if( key->isUp(player+1) || joy->isUp( player+1 ) )
		{
			if( down[player] == 1 )
				down[player] = 2;

			up[player] = 1;
		}
		else
		{
			if( down[player] == 2 )
			{
				down[player] = 1;
			}

			up[player] = 0;
		}

		// BOTTOM
		if( key->isDown(player+1) || joy->isDown( player+1 ) )
		{
			if( up[player] == 1 )
				up[player] = 2;

			down[player] = 1;
		}
		else
		{
			if( up[player] == 2 )
			{
				up[player] = 1;
			}

			down[player] = 0;
		}

		// LEFT
		if( key->isLeft(player+1) || joy->isLeft( player+1 ) )
		{
			if( right[player] == 1 )
				right[player] = 2;

			left[player] = 1;
		}
		else
		{
			if( right[player] == 2 )
			{
				right[player] = 1;
			}

			left[player] = 0;
		}

		// RIGHT
		if( key->isRight(player+1) || joy->isRight( player+1 ) )
		{
			if( left[player] == 1 )
				left[player] = 2;

			right[player] = 1;
		}
		else
		{
			if( left[player] == 2 )
			{
				left[player] = 1;
			}

			right[player] = 0;
		}

		// B1
		if( key->isB1(player+1) || joy->isButton(1,player+1) ) 
		{
			if( b1[player] == 0 )
				b1[player] = 1;
		}
		else
		{
			b1[player] = 0;
		}


		// B1
		if( key->isB2(player+1) || joy->isButton(2,player+1) ) 
		{
			if( b2[player] == 0 )
				b2[player] = 1;
		}
		else
		{
			b2[player] = 0;
		}

		// ESC BUTTON (b3)
		if( key->isEscape() || joy->isButton(3,player) )
		{
			if( exit == 0 )
				exit = 1;
		}
		else
		{
			exit = 0;
		}

	}

    return;
}

/**
 * Return true if the Esc is active
 */
bool ControlsManager::isEscape()
{
	return exit == 1;
}


/**
 * Return true if the Enter key is active
 */
bool ControlsManager::isEnter()
{
	return key->isEnter();
}

/**
 * Return true if the up key is active
 */
bool ControlsManager::isUp	( int player )
{
	return up[ (unsigned int) player-1 ] == 1;
}

/**
 * Return true if the down key is active
 */
bool ControlsManager::isDown	( int player )
{
	return down[ (unsigned int) player-1 ] == 1;
}

/**
 * Return true if the left key is active
 */
bool ControlsManager::isLeft	( int player )
{
	return left[ (unsigned int) player-1 ] == 1;
}

/**
 * Return true if the right key is active
 */
bool ControlsManager::isRight	( int player )
{
	return right[ (unsigned int) player-1 ] == 1;
}

/**
 * Return true if the ok key is active
 */
bool ControlsManager::isB1		( int player )
{
	return b1[ (unsigned int) player-1 ] == 1;
}

/**
 * Return true if the no key is active
 */
bool ControlsManager::isB2		( int player )
{
	return b2[ (unsigned int) player-1 ] == 1;
}

/**
 * Clean the device
 */
void ControlsManager::clean(void)
{
    key->clean();
	joy->clean();
}

#endif
