#ifndef __DirectXManager__
#define __DirectXManager__

#include "Singleton.h"
#include "OptionManager.h"

/**
 * SpritesManager
 *
 * BAUDOUX Jrme - TIXIER Benit
 */
class DirectXManager : public Singleton<DirectXManager>
{
	private:

		friend class Singleton<DirectXManager>; 

	public :

		// WINDOW
		HWND hWnd;

		// D3D
		LPDIRECT3D9				d3d;
		LPDIRECT3DDEVICE9		d3ddev;
		LPDIRECT3DVERTEXBUFFER9 t_buffer;
		LPDIRECT3DSURFACE9		z_buffer;

		// DIN
		LPDIRECTINPUT8			din;
		LPDIRECTINPUTDEVICE8	dinDevice;


		void init(HINSTANCE hInstance,HWND _hWnd);
		bool testMultisampling( int quality );

		~DirectXManager();

	private:
		DirectXManager();


};

/**
 * Private constructor
 */
DirectXManager::DirectXManager()
{
	
}

/**
 * public Destructor
 */
DirectXManager::~DirectXManager()
{
    d3ddev->Release();
    d3d	  ->Release();
}

bool DirectXManager::testMultisampling( int quality )
{
	return SUCCEEDED( d3d->CheckDeviceMultiSampleType(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, D3DFMT_A8R8G8B8 , TRUE, (D3DMULTISAMPLE_TYPE)quality , NULL ) );
}

// this function initializes and prepares Direct3D for use
void DirectXManager::init(HINSTANCE hInstance, HWND _hWnd)
{
	// ---------
	// Win32
	// ---------

	hWnd = _hWnd;

	// ---------
	// D3D
	// ---------

	// OPTIONS
	OptionManager   & op   = OptionManager	::getInstance();

	// D3DINIT
    d3d = Direct3DCreate9(D3D_SDK_VERSION);
    D3DPRESENT_PARAMETERS d3dpp;
    ZeroMemory(&d3dpp, sizeof(d3dpp));


	// FILL THE STRUCT
	if( op.isFullScreen() )
		d3dpp.Windowed				= FALSE;
	else
		d3dpp.Windowed				= TRUE;


    d3dpp.SwapEffect				= D3DSWAPEFFECT_DISCARD;
    d3dpp.hDeviceWindow				= hWnd;
    d3dpp.BackBufferFormat			= D3DFMT_X8R8G8B8;
    d3dpp.BackBufferWidth			= op.getScreenWidth();
    d3dpp.BackBufferHeight			= op.getScreenHeight();
    d3dpp.EnableAutoDepthStencil	= TRUE;	
    d3dpp.AutoDepthStencilFormat	= D3DFMT_D16;

	DWORD q;

	if( ! FAILED( d3d->CheckDeviceMultiSampleType(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, D3DFMT_A8R8G8B8 , TRUE, (D3DMULTISAMPLE_TYPE)op.getMultisampling() , &q ) ) )
	{
		d3dpp.MultiSampleType		= (D3DMULTISAMPLE_TYPE)op.getMultisampling();
		d3dpp.MultiSampleQuality	= q - 1;
	}


    // CREATE DEVICE
    d3d->CreateDevice(D3DADAPTER_DEFAULT,D3DDEVTYPE_HAL,hWnd,D3DCREATE_SOFTWARE_VERTEXPROCESSING,&d3dpp, &d3ddev);

    // Z-BUFFER
	d3ddev->CreateDepthStencilSurface( op.getScreenWidth() ,op.getScreenHeight(), D3DFMT_D16,D3DMULTISAMPLE_NONE, 0,TRUE, &z_buffer, NULL);

    d3ddev->SetRenderState(D3DRS_LIGHTING, TRUE);
    d3ddev->SetRenderState(D3DRS_ZENABLE, TRUE);
    d3ddev->SetRenderState(D3DRS_AMBIENT, D3DCOLOR_XRGB(50, 50, 50));

    // BLENDING
    d3ddev->SetRenderState(D3DRS_ALPHABLENDENABLE,	TRUE);
    d3ddev->SetRenderState(D3DRS_SRCBLEND,			D3DBLEND_SRCALPHA);
    d3ddev->SetRenderState(D3DRS_DESTBLEND,			D3DBLEND_INVSRCALPHA);
    d3ddev->SetRenderState(D3DRS_BLENDOP,			D3DBLENDOP_ADD);



	// ------
	// DIN
	// ------

    DirectInput8Create(hInstance,DIRECTINPUT_VERSION,IID_IDirectInput8,(void**)&din,NULL);
    din->CreateDevice(GUID_SysKeyboard, &dinDevice, NULL);						
    dinDevice->SetDataFormat(&c_dfDIKeyboard);
    dinDevice->SetCooperativeLevel(hWnd,DISCL_NONEXCLUSIVE | DISCL_FOREGROUND );

}


#endif