#ifndef __KeyboardManager__
#define __KeyboardManager__

#include "Singleton.h"
#include <dinput.h>
#include <iostream>
#include <fstream>
#include <sstream>

#define P_EXIT		DIK_ESCAPE
#define P_ENTER		DIK_RETURN

#define P1_UP		DIK_W
#define P1_DOWN		DIK_S
#define P1_LEFT		DIK_A
#define P1_RIGHT	DIK_D
#define P1_B1		DIK_G
#define P1_B2		DIK_H

#define P2_UP		DIK_UP
#define P2_DOWN		DIK_DOWN
#define P2_LEFT		DIK_LEFT
#define P2_RIGHT	DIK_RIGHT
#define P2_B1		DIK_NUMPAD1
#define P2_B2		DIK_NUMPAD2

/**
 * KeyboardManager
 *
 * BAUDOUX Jrme - TIXIER Benit
 */
class KeyboardManager : public Singleton<KeyboardManager>
{
	private:
		friend class Singleton<KeyboardManager>; 

		int exit;
		int pause;
		int enter;

		int up		[2];
		int down	[2];
		int left	[2];
		int right	[2];
		int b1		[2];
		int b2		[2];

		LPDIRECTINPUT8		 din;
		LPDIRECTINPUTDEVICE8 keyBoardDevice;

	public:
		void key();

		inline bool isEscape  (     );
		inline bool isEnter   (     );

		inline bool isUp	  ( int );
		inline bool isDown	  ( int );
		inline bool isLeft	  ( int );
		inline bool isRight   ( int );
		inline bool isB1	  ( int );
		inline bool isB2	  ( int );

		inline void init	  (     );
		inline void init	  ( int );
		inline void initAll	  (     );
		inline void initEscape(     );
		inline void initPause (     );
		inline void initEnter (     );
		inline void initB1	  ( int );
		inline void initB2    ( int );

		void clean();

		~KeyboardManager();

	private:
		KeyboardManager();


};


/**
 * Private constructor
 */
KeyboardManager::KeyboardManager()
{
	initAll();
}

/**
 * Public Destructor
 */
KeyboardManager::~KeyboardManager()
{
	clean();
}

/**
 * Initiate the keyboard
 */
void KeyboardManager::init()
{
	DirectXManager & dx = DirectXManager::getInstance();
	din					= dx.din;
	keyBoardDevice		= dx.dinDevice;

    return; 
}

/**
 * Init all the keys
 */
void KeyboardManager::initAll()
{
	init(1);
	init(2);
}

/**
 * Init the keys of the player
 */
void KeyboardManager::init( int player )
{
	up		[ (unsigned int) player-1 ]	= 3;
	down	[ (unsigned int) player-1 ]	= 3;
	left	[ (unsigned int) player-1 ]	= 3;
	right	[ (unsigned int) player-1 ]	= 3;
	b1		[ (unsigned int) player-1 ]	= 3;
	b2		[ (unsigned int) player-1 ]	= 3;
}

/**
 * Init the Esc button
 */
void KeyboardManager::initEscape()
{
	exit = 3;
}


/**
 * Init the Enter button
 */
void KeyboardManager::initEnter()
{
	enter = 3;	
}

/**
 * Init the Ok button
 */
void KeyboardManager::initB1( int player )
{
	b1		[ (unsigned int) player-1 ]	= 3;
}

/**
 * Init the No button
 */
void KeyboardManager::initB2( int player )
{
	b2		[ (unsigned int) player-1 ]	= 3;	
}

/**
 * Check the keys press
 */
void KeyboardManager::key(void)
{
	// Init
    static BYTE keystate[256];	
	ZeroMemory(keystate, sizeof(keystate) );
    HRESULT hr = keyBoardDevice->GetDeviceState(256, (LPVOID)keystate);

	if (FAILED(hr))
	{
		hr = keyBoardDevice->Acquire();

		while( hr == DIERR_INPUTLOST )    
			 hr = keyBoardDevice->Acquire();

		if (FAILED(hr))
			return;

		keyBoardDevice->GetDeviceState(256, (LPVOID)keystate);
	}


	// ---------------
	// GENERAL
	// ---------------

	// EXIT KEY
    if(keystate[P_EXIT] & 0x80) 
	{
		if( exit == 0 )
			exit = 1;
	}
	else
	{
		if( exit != 0 )
			exit = 0;
	}


	// ENTER
	if(keystate[P_ENTER] & 0x80) 
	{
		if( enter == 0 )
			enter = 1;
	}
	else
	{
		if( enter != 0 )
			enter = 0;
	}

	// ---------------
	// PLAYER 1
	// ---------------

	// UP
	// IF PRESSE THE UP KEY THEN THE KEY IS PRIORITARY
    if(keystate[P1_UP] & 0x80) 
	{
		if( up[0] == 0 )
		{
			if( down[0] == 1 )
			{
				down[0] = 2;
			}

			up[0] = 1;
		}
	}
	// IF REELEASE THE UP KEY THEN WE ACTIVATE THE DOWN KEY THE UP KEY WAS PRIORITIZED
	else
	{
		if( up[0] != 0 )
		{
			if( down[0] == 2 )
			{
				down[0] = 1;
			}

			up[0] = 0;
		}
	}

	// DOWN
	// IF PRESSE THE DOWN KEY THEN THE KEY IS PRIORITARY
    if(keystate[P1_DOWN] & 0x80) 
	{
		if( down[0] == 0 )
		{
			if( up[0] == 1 )
			{
				up[0] = 2;
			}

			down[0] = 1;
		}
	}
	// IF REELEASE THE DOWN KEY THEN WE ACTIVATE THE UP KEY THE DOWN KEY WAS PRIORITIZED
	else
	{
		if( down[0] != 0 )
		{
			if( up[0] == 2 )
			{
				up[0] = 1;
			}

			down[0] = 0;
		}
	}


	//--


	// LEFT
	// IF PRESSE THE LEFT KEY THEN THE KEY IS PRIORITARY
    if(keystate[P1_LEFT] & 0x80) 
	{
		if( left[0] == 0 )
		{
			if( right[0] == 1 )
			{
				right[0] = 2;
			}

			left[0] = 1;
		}
	}
	// IF REELEASE THE LEFT KEY THEN WE ACTIVATE THE RIGHT KEY THE LEFT KEY WAS PRIORITIZED
	else
	{
		if( left[0] != 0 )
		{
			if( right[0] == 2 )
			{
				right[0] = 1;
			}

			left[0] = 0;
		}
	}

	// RIGHT
	// IF PRESSE THE DOWN KEY THEN THE KEY IS PRIORITARY
    if(keystate[P1_RIGHT] & 0x80) 
	{
		if( right[0] == 0 )
		{
			if( left[0] == 1 )
			{
				left[0] = 2;
			}

			right[0] = 1;
		}
	}
	// IF REELEASE THE RIGHT KEY THEN WE ACTIVATE THE LEFT KEY THE RIGHT KEY WAS PRIORITIZED
	else
	{
		if( right[0] != 0 )
		{
			if( left[0] == 2 )
			{
				left[0] = 1;
			}

			right[0] = 0;
		}
	}



	// B1
    if(keystate[P1_B1] & 0x80) 
	{
		if( b1[0] == 0 )
			b1[0] = 1;
	}
	// IF REELEASE
	else
	{
		b1[0] = 0;
	}


	// B1
    if(keystate[P1_B2] & 0x80) 
	{
		if( b2[0] == 0 )
			b2[0] = 1;
	}
	// IF REELEASE
	else
	{
		b2[0] = 0;
	}












	// ---------------
	// PLAYER 2
	// ---------------

	// UP
	// IF PRESSE THE UP KEY THEN THE KEY IS PRIORITARY
    if(keystate[P2_UP] & 0x80) 
	{
		if( up[1] == 0 )
		{
			if( down[1] == 1 )
			{
				down[1] = 2;
			}

			up[1] = 1;
		}
	}
	// IF REELEASE THE UP KEY THEN WE ACTIVATE THE DOWN KEY THE UP KEY WAS PRIORITIZED
	else
	{
		if( up[1] != 0 )
		{
			if( down[1] == 2 )
			{
				down[1] = 1;
			}

			up[1] = 0;
		}
	}

	// DOWN
	// IF PRESSE THE DOWN KEY THEN THE KEY IS PRIORITARY
    if(keystate[P2_DOWN] & 0x80) 
	{
		if( down[1] == 0 )
		{
			if( up[1] == 1 )
			{
				up[1] = 2;
			}

			down[1] = 1;
		}
	}
	// IF REELEASE THE DOWN KEY THEN WE ACTIVATE THE UP KEY THE DOWN KEY WAS PRIORITIZED
	else
	{
		if( down[1] != 0 )
		{
			if( up[1] == 2 )
			{
				up[1] = 1;
			}

			down[1] = 0;
		}
	}


	//--


	// LEFT
	// IF PRESSE THE LEFT KEY THEN THE KEY IS PRIORITARY
    if(keystate[P2_LEFT] & 0x80) 
	{
		if( left[1] == 0 )
		{
			if( right[1] == 1 )
			{
				right[1] = 2;
			}

			left[1] = 1;
		}
	}
	// IF REELEASE THE LEFT KEY THEN WE ACTIVATE THE RIGHT KEY THE LEFT KEY WAS PRIORITIZED
	else
	{
		if( left[1] != 0 )
		{
			if( right[1] == 2 )
			{
				right[1] = 1;
			}

			left[1] = 0;
		}
	}

	// RIGHT
	// IF PRESSE THE DOWN KEY THEN THE KEY IS PRIORITARY
    if(keystate[P2_RIGHT] & 0x80) 
	{
		if( right[1] == 0 )
		{
			if( left[1] == 1 )
			{
				left[1] = 2;
			}

			right[1] = 1;
		}
	}
	// IF REELEASE THE RIGHT KEY THEN WE ACTIVATE THE LEFT KEY THE RIGHT KEY WAS PRIORITIZED
	else
	{
		if( right[1] != 0 )
		{
			if( left[1] == 2 )
			{
				left[1] = 1;
			}

			right[1] = 0;
		}
	}

	// B1
    if(keystate[P2_B1] & 0x80) 
	{
		if( b1[1] == 0 )
			b1[1] = 1;
	}
	// IF REELEASE
	else
	{
		b1[1] = 0;
	}


	// B1
    if(keystate[P2_B2] & 0x80) 
	{
		if( b2[1] == 0 )
			b2[1] = 1;
	}
	// IF REELEASE
	else
	{
		b2[1] = 0;
	}


    return;
}

/**
 * Return true if the Esc is active
 */
bool KeyboardManager::isEscape()
{
	return exit == 1;
}

/**
 * Return true if the Enter key is active
 */
bool KeyboardManager::isEnter()
{
	return enter == 1;
}

/**
 * Return true if the up key is active
 */
bool KeyboardManager::isUp	( int player )
{
	return up[ (unsigned int) player-1 ] == 1;
}

/**
 * Return true if the down key is active
 */
bool KeyboardManager::isDown	( int player )
{
	return down[ (unsigned int) player-1 ] == 1;
}

/**
 * Return true if the left key is active
 */
bool KeyboardManager::isLeft	( int player )
{
	return left[ (unsigned int) player-1 ] == 1;
}

/**
 * Return true if the right key is active
 */
bool KeyboardManager::isRight	( int player )
{
	return right[ (unsigned int) player-1 ] == 1;
}

/**
 * Return true if the ok key is active
 */
bool KeyboardManager::isB1		( int player )
{
	return b1[ (unsigned int) player-1 ] == 1;
}

/**
 * Return true if the no key is active
 */
bool KeyboardManager::isB2		( int player )
{
	return b2[ (unsigned int) player-1 ] == 1;
}

/**
 * Clean the device
 */
void KeyboardManager::clean(void)
{
	if( keyBoardDevice != NULL )
	{
		keyBoardDevice->Unacquire();
		keyBoardDevice->Release();
		keyBoardDevice = NULL;
	}

	if( din != NULL )
	{
		din->Release();
		din = NULL;
	}

}

#endif
