#ifndef __MeshesManager__
#define __MeshesManager__

#include "Singleton.h"
#include "DirectXManager.h"
#include "../Elements/Mesh.h"
#include <iostream>
#include <map>
//#include <atlbase.h>





/**
 * SpritesManager
 *
 * BAUDOUX Jrme - TIXIER Benit
 */
class MeshesManager : public Singleton<MeshesManager>
{
	private:

		friend class Singleton<MeshesManager>; 

		LPDIRECT3DDEVICE9				d3ddev;
		std::map< std::string,Mesh* >	meshes;

	public :

		void		init		( );
		Mesh*		addMesh		( std::string, std::string );
		void		removeMesh	( std::string );
		Mesh*		getMesh		( std::string );


		~MeshesManager();

	private:
		MeshesManager();



};

/**
 * Private constructor
 */
MeshesManager::MeshesManager()
{
	
}

/**
 * public Destructor
 */
MeshesManager::~MeshesManager()
{
	for( std::map<std::string, Mesh* >::const_iterator it = meshes.begin(); it != meshes.end(); ++it )
		if( it->second != NULL )
		{
			delete it->second;
			meshes[it->first] = NULL;
		}
}

/**
 * Init the mesh manager
 */
void MeshesManager::init()
{
	DirectXManager  & dx	= DirectXManager	::getInstance();
	d3ddev					= dx.d3ddev;
}

/**
 * Add a mesh
 * @param name name of the mesh
 * @param path path where is the mesh
 */
Mesh* MeshesManager::addMesh( std::string name, std::string path )
{
	for( std::map<std::string, Mesh* >::const_iterator it = meshes.begin(); it != meshes.end(); ++it )
	{
		if( it->first == name )
			return it->second;
	}

	Mesh * m = new Mesh();

	LPD3DXBUFFER bufMaterial;

	D3DXLoadMeshFromX(	path.c_str(),				// load this file
						D3DXMESH_SYSTEMMEM,			// load the mesh into system memory
						d3ddev,						// the Direct3D Device
						NULL,						// we aren't using adjacency
						&bufMaterial,				// put the materials here
						NULL,						// we aren't using effect instances
						&m->numMaterials,			// the number of materials in this model
						&m->mesh);					// put the mesh here

	// retrieve the pointer to the buffer containing the material information
	D3DXMATERIAL* tempMaterials = (D3DXMATERIAL*)bufMaterial->GetBufferPointer();

	// create a new material buffer and texture for each material in the mesh
	m->material		= new D3DMATERIAL9[m->numMaterials];
	m->texture		= new LPDIRECT3DTEXTURE9[m->numMaterials];

	for(DWORD i = 0; i < m->numMaterials; i++)	
	{
		m->material[i]			= tempMaterials[i].MatD3D;
		m->material[i].Ambient	= m->material[i].Diffuse;

		if( FAILED(	D3DXCreateTextureFromFileEx(  d3ddev, tempMaterials[i].pTextureFilename, D3DX_DEFAULT, D3DX_DEFAULT, D3DX_DEFAULT, D3DPOOL_DEFAULT, D3DFMT_UNKNOWN, D3DPOOL_DEFAULT, D3DX_FILTER_LINEAR, D3DX_FILTER_LINEAR, 0xFFFF00FF, NULL, NULL, &m->texture[i])))
			if( FAILED(D3DXCreateTextureFromFile( d3ddev, tempMaterials[i].pTextureFilename, &m->texture[i]) ) )
				m->texture[i] = NULL;								
	}

	meshes[ name ] = m;

	return m;
}

/**
 * Remove a mesh
 * @param name name of the mesh
 */
void MeshesManager::removeMesh( std::string name )
{
	std::map<std::string, Mesh* >::const_iterator it = meshes.find( name );

	if( it->second != NULL )
		delete it->second;

	meshes.erase( name );
}

/**
 * Get a mesh
 * @param name name of the mesh
 */
Mesh * MeshesManager::getMesh( std::string name )
{
	std::map<std::string, Mesh* >::const_iterator it = meshes.find( name );
	return it->second;
}


#endif