#ifndef __OptionManager__
#define __OptionManager__

#include "Singleton.h"

/**
 * SpritesManager
 *
 * BAUDOUX Jrme - TIXIER Benit
 */
class OptionManager : public Singleton<OptionManager>
{
	private:
		friend class Singleton<OptionManager>; 

		int screenWidth[2];
		int screenHeight[2];
		int fullScreen[2];
		int multisampling[2];

	public :
		void read();

		void save( int width, int height, bool full, int );

		int  getScreenWidth  ( bool temporary );
		int  getScreenHeight ( bool temporary );
		int  getMultisampling( bool temporary );
		bool isFullScreen	 ( bool temporary );

		~OptionManager();

	private:
		OptionManager();



};

/**
 * Private constructor
 */
OptionManager::OptionManager()
{
	screenWidth	 [0] = 640;
	screenHeight [0] = 480;
	fullScreen	 [0] = 0;
	multisampling[0] = 0;

	screenWidth	 [1] = 640;
	screenHeight [1] = 480;
	fullScreen	 [1] = 0;
	multisampling[1] = 0;

	read();
}

/**
 * public Destructor
 */
OptionManager::~OptionManager()
{

}

int OptionManager::getScreenWidth( bool temporary = false )
{
	if( temporary )
		return screenWidth[1];
	return screenWidth[0];
}

int OptionManager::getScreenHeight( bool temporary = false )
{
	if( temporary )
		return screenHeight[1];
	return screenHeight[0];
}

int OptionManager::getMultisampling( bool temporary = false )
{
	if( temporary )
		return multisampling[1];
	return multisampling[0];
}

bool OptionManager::isFullScreen( bool temporary = false )
{
	if( temporary )
		return fullScreen[1] == 1;
	return fullScreen[0] == 1;
}


/**
 * Read the ini file
 */
void OptionManager::read()
{
	char string[128];
	GetPrivateProfileString("Screen", "width","none", string, 128, "./Data/config.ini" ); 
	screenWidth[0] = screenWidth[1]  = atoi(string);

	GetPrivateProfileString("Screen", "height","none", string, 128, "./Data/config.ini" ); 
	screenHeight[0]= screenHeight[1] = atoi(string);

	GetPrivateProfileString("Screen", "fullscreen","none", string, 128, "./Data/config.ini" ); 
	fullScreen[0]  = fullScreen[1]   = atoi(string);

	GetPrivateProfileString("Screen", "multisample","none", string, 128, "./Data/config.ini" ); 
	multisampling[0]  = multisampling[1]   = atoi(string);
}

void OptionManager::save( int width, int height, bool full, int multi = 0 )
{
	char string[128];

	screenWidth  [1] = width;
	screenHeight [1] = height;
	fullScreen   [1] = full;
	multisampling[1] = multi;


	_itoa_s(width,string,10);
	WritePrivateProfileString("Screen","width",string,"./Data/config.ini");

	_itoa_s(height,string,10);
	WritePrivateProfileString("Screen","height",string,"./Data/config.ini");

	_itoa_s(multi,string,10);
	WritePrivateProfileString("Screen","multisample",string,"./Data/config.ini");

	if( full )
		WritePrivateProfileString("Screen","fullscreen",TEXT("1"),"./Data/config.ini");
	else
		WritePrivateProfileString("Screen","fullscreen",TEXT("0"),"./Data/config.ini");


}



#endif