#ifndef __ShaderManager__
#define __ShaderManager__

#include <map>
#include <string>

#include "DirectXManager.h"
#include "Singleton.h"

#define SAFE_RELEASE(x) { if (x!=NULL) { x->Release(); x=NULL; } }
#define SAFE_DELETE(x)  { if (x!=NULL) { delete x; x=NULL; } }
#define SAFE_DELETE_ARRAY(x)  { if (x!=NULL) { delete [] x; x=NULL; } }

// Warnig disable beacause VC++ does not implement unexpected 
#pragma warning( disable : 4290 ) 


class ShaderManager: public Singleton<ShaderManager>
{
private:
	friend class Singleton<ShaderManager>; 
	DirectXManager *        dm ;   // Device

	class myShader
	{
	public:
		LPDIRECT3DVERTEXSHADER9 pVS0;   // Vertex Shader
		LPDIRECT3DPIXELSHADER9  pPS0;   // Pixel Shader
		LPD3DXCONSTANTTABLE     pCVS;   // table des constantes pour le Vertex Shader Null pour asm
		LPD3DXCONSTANTTABLE     pCPS;   // table des constantes pour le Pixel Shader Null pour asm

		myShader() : pVS0(NULL), pPS0(NULL), pCVS(NULL), pCPS(NULL)
		{}
		myShader(const myShader & ms) : pVS0(ms.pVS0), pPS0(ms.pPS0), pCVS(ms.pCVS), pCPS(ms.pCPS)
		{}
		virtual ~myShader()
		{
			SAFE_RELEASE(pVS0);
			SAFE_RELEASE(pPS0);
			SAFE_RELEASE(pCVS);
			SAFE_RELEASE(pCPS);
		}
	};

	std::map<std::string,myShader> shader;

public:
	~ShaderManager();

	void init();
	void addShaderASM(const std::string & nomShader, const char * fileVertex, const char * filePixel) throw(char*);
	void ShaderManager::addShaderHLSL(const std::string & nomShader,
									  const char * fileVertex, const char * mainVertex, const char * ShaderProfileVertex,
									  const char * filePixel, const char * mainPixel, const char * ShaderProfilePixel,
									  LPD3DXINCLUDE pInclude=NULL);

	inline void addVertexBool (UINT numRegistre, CONST BOOL * Variable, UINT nbRegistreTake);
	inline void addVertexInt  (UINT numRegistre, CONST int  * Variable, UINT nbRegistreTake);
	inline void addVertexFloat(UINT numRegistre, CONST float* Variable, UINT nbRegistreTake);

	inline void addPixelBool  (UINT numRegistre, CONST BOOL * Variable, UINT nbRegistreTake);
	inline void addPixelInt   (UINT numRegistre, CONST int  * Variable, UINT nbRegistreTake);
	inline void addPixelFloat (UINT numRegistre, CONST float* Variable, UINT nbRegistreTake);

	inline void activateShader(const std::string & nomShader);
	inline void disableShader ();

private:
	ShaderManager();
};

ShaderManager::ShaderManager()// : pVS0(NULL), pPS0(NULL)
{
	//pDEV = &DirectXManager::getInstance().d3ddev;
	//dxM = &DirectXManager::getInstance();
}

ShaderManager::~ShaderManager()
{
	shader.clear();
}

void ShaderManager::init()
{
	dm = &DirectXManager::getInstance();

	/*Finir init avec un assembleur(ou HLSL) qui fait le minimum Pour l'appeler dans inline void disableShader (); */

	/*
	const char vertex []= "vs_2_x \n\
	dcl_position v0 \n\
	dcl_normal   v1 \n\
	def c8,1.0,.0,.5,.0 \n\
	sub r0, c4,v0 \n\
	nrm r1, r0 \n\
	m4x4 r0,  v0,c0 \n\
	mov oPos, r0 \n\
	mov oT0,v1  \n\
	mov oT1,r1 \n\
	mov oT2,v0  \n\
	dst r0, v0,c4 \n\
	if_lt r0.x,c7.y \n\
	mov r0,c8.y \n\
	else \n\
	mov r0,c8.x \n\
	endif \n\
	mul r1,c6,r0.x \n\
	sub r0,c8.x,r0.x \n\
	mul r0,c5,r0 \n\
	add r0,r0,r1 \n\
	mov oD0,r0 ";
	//if FAILED( D3DXAssembleShader(vertex, sizeof(vertex),NULL,NULL,0,&pShadObj,&pErrMsg) ) 
	*/

}

void ShaderManager::addShaderHLSL(const std::string & nomShader,
								  const char * fileVertex, const char * mainVertex, const char * ShaderProfileVertex,
								  const char * filePixel, const char * mainPixel, const char * ShaderProfilePixel,
								  LPD3DXINCLUDE pInclude)
{
	myShader * ms = new myShader;
	LPD3DXBUFFER    pShadObj, pErrMsg;

	//////////////////////////////////
	// compilation du vertex shader //
	//////////////////////////////////
	if FAILED( D3DXCompileShaderFromFile(fileVertex,NULL,pInclude,mainVertex,ShaderProfileVertex,0,&pShadObj,&pErrMsg,&ms->pCVS) ) 
	{
		char *ErrMsg = (char *)pErrMsg->GetBufferPointer();
		MessageBox(NULL, ErrMsg,"HLSL compilation error" , MB_OK);
	}
	SAFE_RELEASE(pErrMsg);

	// creation du vertex shader
	dm->d3ddev->CreateVertexShader( (DWORD *)pShadObj->GetBufferPointer(), &ms->pVS0 );
	SAFE_RELEASE(pShadObj);

	/////////////////////////////////
	// compilation du pixel shader //
	/////////////////////////////////
	if FAILED( D3DXCompileShaderFromFile(filePixel,NULL,pInclude,mainPixel,ShaderProfilePixel,0,&pShadObj,&pErrMsg,&ms->pCPS) ) 
	{
		char *ErrMsg = (char *)pErrMsg->GetBufferPointer();
		MessageBox(NULL, ErrMsg,"HLSL compilation error" , MB_OK);
	}
	SAFE_RELEASE(pErrMsg);

	// creation du vertex shader
	dm->d3ddev->CreatePixelShader( (DWORD *)pShadObj->GetBufferPointer(), &ms->pPS0 );
	SAFE_RELEASE(pShadObj);

	// fixe les shaders
	shader[nomShader] = *ms;
}

void ShaderManager::addShaderASM(const std::string & nomShader,const char * fileVertex, const char * filePixel)
{
	myShader * ms = new myShader;
	LPD3DXBUFFER    pShadObj, pErrMsg;
	char			*ErrMsg;

	//////////////////////////////////
	// compilation du vertex shader //
	//////////////////////////////////

	if FAILED( D3DXAssembleShaderFromFile(fileVertex, NULL,NULL,0,&pShadObj,&pErrMsg) ) 
	{
		ErrMsg = (char *)pErrMsg->GetBufferPointer();
		// MessageBox(NULL, ErrMsg,"VS compilation error" , MB_OK);	
		throw( ErrMsg );
	}
	SAFE_RELEASE(pErrMsg);

	// creation du vertex shader
	dm->d3ddev->CreateVertexShader( (DWORD *)pShadObj->GetBufferPointer(), &ms->pVS0 );
	SAFE_RELEASE(pShadObj);

	/////////////////////////////////
	// compilation du pixel shader //
	/////////////////////////////////
	if FAILED( D3DXAssembleShaderFromFile(filePixel, NULL,NULL,0,&pShadObj,&pErrMsg) ) 
	{
		ErrMsg = (char *)pErrMsg->GetBufferPointer();
		// MessageBox(NULL, ErrMsg,"VS compilation error" , MB_OK);
		throw( ErrMsg );
	}
	SAFE_RELEASE(pErrMsg);

	// creation du vertex shader
	dm->d3ddev->CreatePixelShader( (DWORD *)pShadObj->GetBufferPointer(), &ms->pPS0 );
	SAFE_RELEASE(pShadObj);

	// fixe les shaders
	shader[nomShader] = *ms;
}

//
// Fonction pour attribuer les constantes au vertex
//
void ShaderManager::addVertexBool(UINT numRegistre,CONST BOOL* Variable,UINT nbRegistreTake)
{
	dm->d3ddev->SetVertexShaderConstantB(numRegistre, Variable, nbRegistreTake);
}

void ShaderManager::addVertexInt(UINT numRegistre,CONST int* Variable,UINT nbRegistreTake)
{
	dm->d3ddev->SetVertexShaderConstantI(numRegistre, Variable, nbRegistreTake);
}

void ShaderManager::addVertexFloat(UINT numRegistre,CONST float* Variable,UINT nbRegistreTake)
{
	dm->d3ddev->SetVertexShaderConstantF(numRegistre, Variable, nbRegistreTake);
}

//
// Fonction pour attribuer les constantes au pixel
//
void ShaderManager::addPixelBool(UINT numRegistre,CONST BOOL* Variable,UINT nbRegistreTake)
{
	dm->d3ddev->SetPixelShaderConstantB(numRegistre, Variable, nbRegistreTake);
}

void ShaderManager::addPixelInt(UINT numRegistre,CONST int* Variable,UINT nbRegistreTake)
{
	dm->d3ddev->SetPixelShaderConstantI(numRegistre, Variable, nbRegistreTake);
}

void ShaderManager::addPixelFloat(UINT numRegistre,CONST float* Variable,UINT nbRegistreTake)
{
	dm->d3ddev->SetPixelShaderConstantF(numRegistre, Variable, nbRegistreTake);
}

void ShaderManager::activateShader(const std::string & nomShader)
{
	myShader & ms = shader[nomShader];
	// fixe les shaders
	dm->d3ddev->SetVertexShader(ms.pVS0);
	dm->d3ddev->SetPixelShader (ms.pPS0);
}

void ShaderManager::disableShader ()
{

}

#endif
