#ifndef __SoundsManager__
#define __SoundsManager__

#include "Singleton.h"
#include "../Elements/Sound.h"
#include "../Elements/Music.h"

#include <iostream>
#include <map>
#include <vector>
#include <string>

#include <al.h>
#include <alc.h>


/**
 * SpritesManager
 *
 * BAUDOUX Jrme - TIXIER Benot
 */
class SoundsManager : public Singleton<SoundsManager>
{
	private:
		friend class Singleton<SoundsManager>; 
		std::map< std::string,Sound* >	sounds;
		std::map< std::string,Music* >	musics;

	public :

		bool		init		( );
		Sound*		addSound	( std::string, std::string, int, bool );
		void		removeSound	( std::string );
		Sound*		getSound	( std::string );

		Music*		addMusic	( std::string, std::string );
		void		removeMusic	( std::string );
		Music*		getMusic	( std::string );

		void		refresh();

		~SoundsManager();

	private:
		SoundsManager();



};

/**
 * Private constructor
 */
SoundsManager::SoundsManager()
{

}

/**
 * public Destructor
 */
SoundsManager::~SoundsManager()
{
	for( std::map<std::string, Sound*>::const_iterator it = sounds.begin(); it != sounds.end(); ++it )
	{
		if( it->second != NULL )
		{
			delete it->second;
			sounds[it->first] = NULL;
		}
	}

	for( std::map<std::string, Music*>::const_iterator it = musics.begin(); it != musics.end(); ++it )
	{
		if( it->second != NULL )
		{
			delete it->second;
			sounds[it->first] = NULL;
		}
	}

    // Rcupration du contexte et du device
    ALCcontext* Context = alcGetCurrentContext();
    ALCdevice*  Device  = alcGetContextsDevice(Context);

    // Dsactivation du contexte
    alcMakeContextCurrent(NULL);

    // Destruction du contexte
    alcDestroyContext(Context);

    // Fermeture du device
    alcCloseDevice(Device);
}



/**
 * Init the sound manager
 */
bool SoundsManager::init()
{
    // GET ALL THE DEVICES
    const ALCchar* DeviceList = alcGetString(NULL, ALC_DEVICE_SPECIFIER);

    if ( strlen(DeviceList) > 0 )
    {
		// OPEN THE FIRST DEVICE
		ALCdevice* Device = alcOpenDevice(DeviceList);
		if (!Device)
			return false;

		// CONTEXT CREATION
		ALCcontext* Context = alcCreateContext(Device, NULL);
		if (!Context)
			return false;

		// CONTEXT ACTIVATION
		if (!alcMakeContextCurrent(Context))
			return false;
    }
	else
		return false;

	return true;
}




/**
 * Add a sound
 * @param name name of the sound
 * @param path path where is the sound
 * @param nbEch number of sources
 * @param loop is it looping
 * @return the sound
 */
Sound* SoundsManager::addSound( std::string name, std::string path, int nbEch = 1, bool loop=false )
{
	for( std::map<std::string, Sound* >::const_iterator it = sounds.begin(); it != sounds.end(); ++it )
	{
		if( it->first == name )
			return it->second;
	}

	Sound * sound = new Sound(path,nbEch,loop);
	sounds[ name ] = sound;
	return sound;
}

/**
 * Add a music
 * @param name name of the music
 * @param path path where is the music
 * @return the music
 */
Music* SoundsManager::addMusic( std::string name, std::string path )
{
	for( std::map<std::string, Music* >::const_iterator it = musics.begin(); it != musics.end(); ++it )
	{
		if( it->first == name )
			return it->second;
	}

	Music * music = new Music(path);
	musics[ name ] = music;
	return music;
}

/**
 * Remove a sound
 * @param name name of the sound
 */
void SoundsManager::removeSound( std::string name )
{
	std::map<std::string, Sound* >::const_iterator it = sounds.find( name );

	if( it->second != NULL )
		delete it->second;

	sounds.erase( name );
}

/**
 * Remove a music
 * @param name name of the music
 */
void SoundsManager::removeMusic( std::string name )
{
	std::map<std::string, Music* >::const_iterator it = musics.find( name );

	if( it->second != NULL )
		delete it->second;

	musics.erase( name );
}

/**
 * Get a sound
 * @param name name of the sound
 * @return sound
 */
Sound * SoundsManager::getSound( std::string name )
{
	std::map<std::string, Sound* >::const_iterator it = sounds.find( name );
	return it->second;
}

/**
 * Get a music
 * @param name name of the music
 * @return music
 */
Music * SoundsManager::getMusic( std::string name )
{
	std::map<std::string, Music* >::const_iterator it = musics.find( name );
	return it->second;
}

/**
 * Refresh all the musics
 */
void SoundsManager::refresh()
{
	//for( std::map<std::string, Music* >::const_iterator it = musics.begin(); it != musics.end(); ++it )
	//	it->second->refresh();
}

#endif