#ifndef __SpritesManager__
#define __SpritesManager__

#include "Singleton.h"
#include "DirectXManager.h"
#include <iostream>
#include <map>
#include <string>


/**
 * SpritesManager
 *
 * BAUDOUX Jrme - TIXIER Benit
 */
class SpritesManager : public Singleton<SpritesManager>
{
	private:

		friend class Singleton<SpritesManager>; 

		LPDIRECT3DDEVICE9							d3ddev;
		LPD3DXSPRITE								d3dspt;
		std::map< std::string,LPDIRECT3DTEXTURE9 >	sprites;

	public :

		void					init		( );
		LPDIRECT3DTEXTURE9		addSprite	( std::string, std::string );
		void					removeSprite( std::string );
		LPDIRECT3DTEXTURE9		getSprite	( std::string );

		LPD3DXSPRITE			getSpriteDevice();
		void					drawSprite( std::string, float, float, float, float, float, float );

		~SpritesManager();

	private:
		SpritesManager();



};

/**
 * Private constructor
 */
SpritesManager::SpritesManager()
{
	
}

/**
 * public Destructor
 */
SpritesManager::~SpritesManager()
{
	for( std::map<std::string, LPDIRECT3DTEXTURE9>::const_iterator it = sprites.begin(); it != sprites.end(); ++it )
		if( it->second != NULL )
			it->second->Release();
}

void SpritesManager::init( )
{
	DirectXManager  & dx   = DirectXManager	::getInstance();
	d3ddev = dx.d3ddev;
	D3DXCreateSprite(d3ddev, &d3dspt);
}

inline LPD3DXSPRITE SpritesManager::getSpriteDevice()
{
	return d3dspt;
}

LPDIRECT3DTEXTURE9 SpritesManager::addSprite( std::string name, std::string path )
{
	for( std::map<std::string, LPDIRECT3DTEXTURE9>::const_iterator it = sprites.begin(); it != sprites.end(); ++it )
	{
		if( it->first == name )
			return it->second;
	}

	LPDIRECT3DTEXTURE9 sptemp = NULL;
	D3DXCreateTextureFromFile(d3ddev, path.c_str(), &sptemp);
	sprites[ name ] = sptemp;

	return sptemp;
}

void SpritesManager::removeSprite( std::string name )
{
	sprites.erase( name );
}

LPDIRECT3DTEXTURE9 SpritesManager::getSprite( std::string name )
{
	std::map<std::string, LPDIRECT3DTEXTURE9>::const_iterator it = sprites.find( name );
	return it->second;
}

void SpritesManager::drawSprite( std::string name, float cx, float cy, float cz, float px, float py, float pz )
{
	std::map<std::string, LPDIRECT3DTEXTURE9>::const_iterator it = sprites.find( name );

	D3DXVECTOR3 center	(cx, cy, cz);
	D3DXVECTOR3 position(px, py, pz);
	d3dspt->Draw( it->second, NULL, &center, &position, D3DCOLOR_XRGB(255, 255, 255));
}

#endif