#ifndef __TextManager__
#define __TextManager__

#include "Singleton.h"
#include "DirectXManager.h"
#include <iostream>
#include <map>


/**
 * SpritesManager
 *
 * BAUDOUX Jrme - TIXIER Benit
 */
class TextManager : public Singleton<TextManager>
{
	private:

		friend class Singleton<TextManager>; 

		LPDIRECT3DDEVICE9							d3ddev;
		std::map< std::string,LPD3DXFONT  >			fonts;

	public :

		void					init		( );
		void					addFont		( std::string, std::string, int, int, bool, int );
		void					removeFont	( std::string );
		LPD3DXFONT				getFont		( std::string );

		void					drawText(  std::string font, std::string text, int x, int y, int xx, int yy, int r, int g, int b, int a, DWORD options );

		~TextManager();

	private:
		TextManager();



};

/**
 * Private constructor
 */
TextManager::TextManager()
{
	
}

/**
 * public Destructor
 */
TextManager::~TextManager()
{
	for( std::map<std::string, LPD3DXFONT>::const_iterator it = fonts.begin(); it != fonts.end(); ++it )
		if( it->second != NULL )
			it->second->Release();
}

void TextManager::init()
{
	DirectXManager  & dx	= DirectXManager	::getInstance();
	d3ddev					= dx.d3ddev;
}


void TextManager::addFont( std::string name, std::string font, int size, int width, bool italic, int weight )
{
	bool alreadyExists = false;

	for( std::map<std::string, LPD3DXFONT>::const_iterator it = fonts.begin(); it != fonts.end() && ! alreadyExists; ++it )
	{
		if( it->first == name )
			alreadyExists = true;
	}

	if( !alreadyExists )
	{
		LPD3DXFONT fontTemp = NULL;

		D3DXCreateFont(d3ddev,							// the D3D Device
					   size,							// font height of 30
					   width,							// default font width
					   weight,							// font weight
					   0,								// not using MipLevels
					   italic,							// italic font
					   DEFAULT_CHARSET,					// default character set
					   OUT_DEFAULT_PRECIS,				// default OutputPrecision,
					   DEFAULT_QUALITY,					// default Quality
					   DEFAULT_PITCH | FF_DONTCARE,		// default pitch and family
					   font.c_str(),					// use Facename Arial
					   &fontTemp);						// the font object

		fonts[name] = fontTemp;
	}
}

void TextManager::removeFont( std::string name )
{
	fonts.erase( name );
}

LPD3DXFONT TextManager::getFont( std::string name )
{
	std::map<std::string, LPD3DXFONT>::const_iterator it = fonts.find( name );
	return it->second;
}

void TextManager::drawText( std::string font, std::string text, int x, int y, int xx, int yy, int r, int g, int b, int a, DWORD options = DT_LEFT | DT_TOP  )
{
	std::map<std::string, LPD3DXFONT>::const_iterator it = fonts.find( font );

    RECT textbox; 
	SetRect(&textbox, x, y, xx, yy); 

	it->second->DrawTextA(NULL,text.c_str(),(int)text.length(),&textbox,options,D3DCOLOR_ARGB(a, r, g, b));
}

#endif