#ifndef __MENU__
#define __MENU__

#include <iostream>

#include "../Managers/ControlsManager.h"
#include "../Managers/SpritesManager.h"
#include "../Managers/TextManager.h"
#include "../Managers/OptionManager.h"
#include "../Managers/SoundsManager.h"
#include "../Elements/Object2D.h"

#define MENU_TIME_BETWEEN_TWO_KEYS	400
#define MENU_CONTINUE				0

#define MENU_SIZE_FONT				20
#define MENU_SIZE_TITLE				15

/**
 * Menu
 *
 * BAUDOUX Jrme - TIXIER Benot
 */
class Menu : public Object2D
{
	protected:


		// MENU
		int		state;
		int		action;
		int		nbElements;
		float	timeLastKey;

		std::string name;
		std::string * elements;

		// RESSOURCES
		std::string font;
		std::string fontTitle;
		Sound* beep;
		Sound* beep2;

		// MANAGERS
		OptionManager   * op ;
		TextManager     * text;
		SpritesManager  * sp;
		ControlsManager * ctrl;
		SoundsManager   * snd;


	public:
		Menu();
		~Menu();

		virtual void reset();
		virtual std::string getName();
		virtual int  getCode();
		virtual void refresh( long );
		virtual void draw2D();
		virtual void draw();

	protected:
};

/**
 * Constructor
 */
Menu::Menu() : Object2D()
{
	op   = &OptionManager  ::getInstance();
	text = &TextManager	   ::getInstance();
	sp   = &SpritesManager ::getInstance();
	ctrl = &ControlsManager::getInstance();
	snd  = &SoundsManager  ::getInstance();

	width				= op->getScreenWidth();
	height				= op->getScreenHeight();

	timeLastKey			= MENU_TIME_BETWEEN_TWO_KEYS;
	state				= 1;
	action				= MENU_CONTINUE;
	name				= std::string("Menu");
	elements			= NULL;

	// SPRITES
	sprite = sp->addSprite("menu"	,"Images/Menu/menu.png");

	// FONTS
	font = std::string("arialMenu");
	text->addFont(font ,"Arial", op->getScreenHeight()/MENU_SIZE_FONT ,0,false,FW_NORMAL);
	fontTitle = std::string("arialTitle");
	text->addFont(fontTitle ,"Arial", op->getScreenHeight()/MENU_SIZE_TITLE ,0,false,FW_NORMAL);
	
	// SOUNDS
	beep  = snd->addSound("beep" ,"Sounds/beep.wav" ,5);
	beep2 = snd->addSound("beep2","Sounds/beep2.wav",5);
}

/**
 * Destructor
 */
Menu::~Menu()
{

}

/**
 * Name of the menu
 */
std::string Menu::getName()
{
	return name;
}

/**
 * Code 
 */
int Menu::getCode()
{
	return action;
}

void Menu::reset()
{
	
}

/**
 * Refresh the menu
 */
void Menu::refresh( long time )
{

	if( timeLastKey < MENU_TIME_BETWEEN_TWO_KEYS )
	{
		timeLastKey += time;

		if( ! ctrl->isUp(1) && ! ctrl->isDown(1) && ! ctrl->isLeft(1) && ! ctrl->isRight(1) && ! ctrl->isUp(2) && ! ctrl->isDown(2) && ! ctrl->isLeft(2) && ! ctrl->isRight(2) )
			timeLastKey = MENU_TIME_BETWEEN_TWO_KEYS;
	}
	else
	{
		if( ctrl->isUp(1) || ctrl->isUp(2) )
		{
			state --;

			if( state <= 0 )
				state = nbElements;

			timeLastKey = 0;

			beep->play();
		}
		else if( ctrl->isDown(1) || ctrl->isDown(2) )
		{
			state ++;

			if( state > nbElements )
				state = 1;

			timeLastKey = 0;

			beep->play();
		}
	}

	if( ctrl->isB1(1) || ctrl->isB1(2) || ctrl->isEnter() )
	{
		ctrl->initB1(1);
		ctrl->initB1(2);
		ctrl->initEnter();

		action = state;

		beep2->play();
	}
}

/**
 * Draw the menu
 */
void Menu::draw2D()
{
	Object2D::draw();
}

/**
 * Draw the menu
 */
void Menu::draw()
{
	// TITLE OF THE MENU
	text->drawText(	fontTitle,
						name, 
						(int)(op->getScreenWidth() /2.0 - op->getScreenWidth() /3.0) ,
						(int)(op->getScreenHeight()/4.0) , 
						(int)(op->getScreenWidth() /2.0 + op->getScreenWidth() /3.0) ,
						(int)(op->getScreenHeight()/4.0 + op->getScreenHeight()/MENU_SIZE_TITLE) ,
						128,128,255,255, 
						DT_CENTER | DT_VCENTER);


	// WRITE THE ELEMENTS OF TEH MENU
	for( int i=0; i<nbElements; i++ )
	{
		int r= 255 , g= 255, b= 255, a= 255;

		if( i + 1 == state )
		{
			g = 0;
			b = 0;
		}

		text->drawText(	font	,
						elements[i], 
						(int)(op->getScreenWidth() /2.0 - op->getScreenWidth() /3.0) ,
						(int)(op->getScreenHeight()/3.0 + op->getScreenHeight()/MENU_SIZE_FONT*1.5*i) , 
						(int)(op->getScreenWidth() /2.0 + op->getScreenWidth() /3.0) ,
						(int)(op->getScreenHeight()/3.0 + op->getScreenHeight()/MENU_SIZE_FONT*1.5*i + op->getScreenHeight()/MENU_SIZE_FONT) ,
						r,g,b,a, 
						DT_CENTER | DT_VCENTER);
	}
}

#endif
