#ifndef __MENUCHARACTERS__
#define __MENUCHARACTERS__

#include "Menu.h"

enum
{
	MENUCHARACTERS_CONTINUE,
	MENUCHARACTERS_SPEED,
	MENUCHARACTERS_STRONG,
	MENUCHARACTERS_BACK
};

/**
 * Menu
 *
 * BAUDOUX Jrme - TIXIER Benot
 */
class MenuCharacters : public Menu
{
	protected:
		int		state2;
		float	timeLastKey2;
		int		action1;
		int		action2;

	public:
		MenuCharacters();
		~MenuCharacters();
		void refresh(long);
		void draw();

	protected:
};

/**
 * Constructor
 */
MenuCharacters::MenuCharacters() : Menu()
{
	nbElements = 3;

	elements = new std::string[nbElements];

	sprite = sp->addSprite("menuCharacters"	,"Images/Menu/menuCharacters.png");

	elements[0] = "";
	elements[1] = "";
	elements[2] = "Go Back";
	name		= "Character";
	state2		= 1;
	action1		= -1;
	action2		= -1;
}

/**
 * Destructor
 */
MenuCharacters::~MenuCharacters()
{

}

/**
 * Refresh the menu
 */
void MenuCharacters::refresh( long time )
{

	// PLAYER ONE
	if( timeLastKey < MENU_TIME_BETWEEN_TWO_KEYS )
	{
		timeLastKey += time;

		if( ! ctrl->isUp(1) && ! ctrl->isDown(1) && ! ctrl->isLeft(1) && ! ctrl->isRight(1) )
			timeLastKey = MENU_TIME_BETWEEN_TWO_KEYS;
	}
	else if( action1 == -1 )
	{
		if( ctrl->isUp(1) )
		{
			state --;

			if( state <= 0 )
				state = nbElements;

			timeLastKey = 0;

			beep->play();
		}
		else if( ctrl->isDown(1)  )
		{
			state ++;

			if( state > nbElements )
				state = 1;

			timeLastKey = 0;

			beep->play();
		}
	}

	// PLAYER 2
	if( timeLastKey2 < MENU_TIME_BETWEEN_TWO_KEYS )
	{
		timeLastKey2 += time;

		if( ! ctrl->isUp(2) && ! ctrl->isDown(2) && ! ctrl->isLeft(2) && ! ctrl->isRight(2) )
			timeLastKey2 = MENU_TIME_BETWEEN_TWO_KEYS;
	}
	else if( action2 == -1 )
	{
		if(  ctrl->isUp(2) )
		{
			state2 --;

			if( state2 <= 0 )
				state2 = nbElements;

			timeLastKey2 = 0;

			beep->play();
		}
		else if( ctrl->isDown(2) )
		{
			state2 ++;

			if( state2 > nbElements )
				state2 = 1;

			timeLastKey2 = 0;

			beep->play();
		}
	}

	// Si le joueur 1  choisi
	if( ctrl->isB1(1) )
	{
		ctrl->initB1(1);

		//IF THE ACTION IS TO GO BACK TO THE MAIN MENU
		if( state == MENUCHARACTERS_BACK )
			action = state;
		//OTHERWISE
		else
			action1 = state;

		beep2->play();
	}

	// Si le joueur 2  choisi
	if( ctrl->isB1(2) )
	{
		ctrl->initB1(2);

		//IF THE ACTION IS TO GO BACK TO THE MAIN MENU
		if( state2 == MENUCHARACTERS_BACK )
			action = state2;
		//OTHERWISE
		else
			action2 = state2;

		beep2->play();
	}

	// Si les deux joueurs ont choisi code leur choix dans un int
	if( action1 != -1 && action2 != -1 )
		action = action1 + action2*10;
}

/**
 * Draw the menu
 */
void MenuCharacters::draw()
{
	// TITLE OF THE MENU
	text->drawText(	fontTitle,
						name, 
						(int)(op->getScreenWidth() /2.0 - op->getScreenWidth() /3.0) ,
						(int)(op->getScreenHeight()/4.0) , 
						(int)(op->getScreenWidth() /2.0 + op->getScreenWidth() /3.0) ,
						(int)(op->getScreenHeight()/4.0 + op->getScreenHeight()/MENU_SIZE_TITLE) ,
						128,128,255,255, 
						DT_CENTER | DT_VCENTER);


	// WRITE THE ELEMENTS OF TEH MENU
	for( int i=0; i<nbElements; i++ )
	{
		if( i + 1 == state )
		{
			text->drawText(	font	,
						"P1", 
						(int)(op->getScreenWidth() /2 - op->getScreenWidth() /3.0 + 150) ,
						(int)(op->getScreenHeight()/2.5 + op->getScreenHeight()/MENU_SIZE_FONT*3*i) , 
						(int)(op->getScreenWidth() /2 + op->getScreenWidth() /3.0) ,
						(int)(op->getScreenHeight()/2.5 + op->getScreenHeight()/MENU_SIZE_FONT*3*i + op->getScreenHeight()/MENU_SIZE_FONT) ,
						255,0,0,255, 
						DT_LEFT | DT_VCENTER);
		}

		if( i + 1 == state2 )
		{
			text->drawText(	font	,
						"P2", 
						(int)(op->getScreenWidth() /2.0 - op->getScreenWidth() /3.0) ,
						(int)(op->getScreenHeight()/2.5 + op->getScreenHeight()/MENU_SIZE_FONT*3*i) , 
						(int)(op->getScreenWidth() /2.0 + op->getScreenWidth() /3.0 - 150) ,
						(int)(op->getScreenHeight()/2.5 + op->getScreenHeight()/MENU_SIZE_FONT*3*i + op->getScreenHeight()/MENU_SIZE_FONT) ,
						0,0,255,255, 
						DT_RIGHT | DT_VCENTER);
		}

		text->drawText(	font	,
						elements[i], 
						(int)(op->getScreenWidth() /2.0 - op->getScreenWidth() /3.0) ,
						(int)(op->getScreenHeight()/2.5 + op->getScreenHeight()/MENU_SIZE_FONT*3*i) , 
						(int)(op->getScreenWidth() /2.0 + op->getScreenWidth() /3.0) ,
						(int)(op->getScreenHeight()/2.5 + op->getScreenHeight()/MENU_SIZE_FONT*3*i + op->getScreenHeight()/MENU_SIZE_FONT) ,
						255,255,255,255, 
						DT_CENTER | DT_VCENTER);
	}
}

#endif
