#ifndef __MENUINTRO__
#define __MENUINTRO__

#include "Menu.h"

#define MENUINTRO_GO_PRINCIPAL				1
#define MENUINTRO_FADE_TIME					3000.0
#define MENUINTRO_INTRO_TIME				6000.0

/**
 * Menu
 *
 * BAUDOUX Jrme - TIXIER Benot
 */
class MenuIntro : public Menu
{
	protected:
		std::string smallFont;

		long timeRemaining;

		LPD3DXSPRITE		d3dspt;

		LPDIRECT3DTEXTURE9 sprite2;
		LPDIRECT3DTEXTURE9 sprite3;

	public:
		MenuIntro();
		~MenuIntro();
		virtual void refresh( long time );
		virtual void draw2D();
		virtual void draw();

	protected:
};

/**
 * Constructor
 */
MenuIntro::MenuIntro() : Menu()
{
	// GET THE DEVICE
	d3dspt = sp->getSpriteDevice();

	// FILL THE INFOS
	nbElements	= 1;
	elements	= new std::string[nbElements];
	elements[0] = "";
	name		= "MenuIntro";
	timeRemaining	=	(int)MENUINTRO_INTRO_TIME;

	// ADD THE TWO SPRITES
	sp->addSprite("intro1","Images/Menu/intro1.png");
	sprite = sp->getSprite("intro1");
	sp->addSprite("intro2","Images/Menu/intro2.png");
	sprite2 = sp->getSprite("intro2");
	sp->addSprite("intro3","Images/Menu/intro3.png");
	sprite3 = sp->getSprite("intro3");
}

/**
 * Destructor
 */
MenuIntro::~MenuIntro()
{

}

/**
 * Refresh the menu
 */
void MenuIntro::refresh( long time )
{
	// REFRESH
	if( ctrl->isB1(1) || ctrl->isB1(2) || ctrl->isEnter() )
	{
		ctrl->initB1(1);
		ctrl->initB1(2);
		ctrl->initEnter();
		action = state;
		beep2->play();
	}

	// SET THE TIME
	timeRemaining -= time;

	// IF TIME IS UP GO TO THE MAIN MENU
	if( timeRemaining <= 0 )
		action = MENUINTRO_GO_PRINCIPAL;
}

void MenuIntro::draw2D()
{
	//
	// FIRST IMAGE
	//

	// SET THE ALPHA VALUE
	int alpha;
	
	if( timeRemaining - (MENUINTRO_INTRO_TIME-MENUINTRO_FADE_TIME)  > MENUINTRO_FADE_TIME/2 )
		alpha = 255;
	else
		alpha = 0;

	D3DXVECTOR2 spriteCentre=D3DXVECTOR2(0,0);
	D3DXVECTOR2 trans=D3DXVECTOR2((float)x,(float)y);
	D3DXMATRIX mat;
	D3DXVECTOR2 scaling( (float)width/1024.0f, (float)height/1024.0f );
	D3DXMatrixTransformation2D(&mat,NULL,0.0,&scaling,&spriteCentre,0,&trans);
	d3dspt->SetTransform(&mat);
	d3dspt->Draw( sprite, NULL, NULL, NULL, D3DCOLOR_ARGB(alpha, 255, 255, 255));


	//
	// SECOND IMAGE
	//



	// SET THE ALPHA VALUE

	if( timeRemaining - (MENUINTRO_INTRO_TIME-MENUINTRO_FADE_TIME) < MENUINTRO_FADE_TIME/2 )
		alpha = 255;
	else if( timeRemaining > MENUINTRO_FADE_TIME/2 )
		alpha = (int)(256 - (timeRemaining - (MENUINTRO_INTRO_TIME-MENUINTRO_FADE_TIME) - MENUINTRO_FADE_TIME/2) / MENUINTRO_FADE_TIME * 2 * 256);


	if( alpha < 0 )
		alpha = 0;

	if( alpha > 255 )
		alpha = 255;

	spriteCentre=D3DXVECTOR2(0,0);
	trans=D3DXVECTOR2((float)x,(float)y);
	scaling = D3DXVECTOR2( (float)width/1024.0f, (float)height/1024.0f );

	D3DXMatrixTransformation2D(&mat,NULL,0.0,&scaling,&spriteCentre,0,&trans);
	d3dspt->SetTransform(&mat);
	d3dspt->Draw( sprite2, NULL, NULL, NULL, D3DCOLOR_ARGB( alpha, 255, 255, 255));



	//
	// THIRD IMAGE
	//

	
	if( timeRemaining < MENUINTRO_FADE_TIME )
	{
		alpha = 255;
	}
	else if( timeRemaining - (MENUINTRO_INTRO_TIME-MENUINTRO_FADE_TIME) < MENUINTRO_FADE_TIME/2 )
	{
		// SET THE ALPHA VALUE
		alpha = (int)(256 - (timeRemaining - (MENUINTRO_INTRO_TIME-MENUINTRO_FADE_TIME) - MENUINTRO_FADE_TIME/2) / MENUINTRO_FADE_TIME * 2 * 256);
		if( alpha < 0 )
			alpha = 0;
	}
	else
	{
		alpha = 0;
	}

	spriteCentre=D3DXVECTOR2(0,0);
	trans=D3DXVECTOR2((float)x,(float)y);
	scaling = D3DXVECTOR2( (float)width/1024.0f, (float)height/1024.0f );

	D3DXMatrixTransformation2D(&mat,NULL,0.0,&scaling,&spriteCentre,0,&trans);
	d3dspt->SetTransform(&mat);
	d3dspt->Draw( sprite3, NULL, NULL, NULL, D3DCOLOR_ARGB(alpha, 255, 255, 255));
}

void MenuIntro::draw()
{


}

#endif
