#ifndef __MENUOPTIONS__
#define __MENUOPTIONS__

#include "Menu.h"

#define MENUOPTIONS_BACK	4
#define NBRESULTIONS		8


/**
 * Menu
 *
 * BAUDOUX Jrme - TIXIER Benot
 */
class MenuOptions : public Menu
{
	protected:
		int resolutionsWidth[NBRESULTIONS];
		int resolutionsHeight[NBRESULTIONS];

		int  currentResolution;
		int  multisampling;
		bool fullscreen;

	public:
		MenuOptions();
		~MenuOptions();

		void refresh( long time );
		void draw();

	protected:

		void load();
};

/**
 * Constructor
 */
MenuOptions::MenuOptions() : Menu()
{
	// MENU
	nbElements = 4;

	elements = new std::string[nbElements];

	elements[0] = "Resolution";
	elements[1] = "FullScreen";
	elements[2] = "Antialiasing";
	elements[3] = "Return to main menu";

	name		= "Options";

	// LISTS ALL THE RESOLUTIONS
	resolutionsWidth[0] = 640;
	resolutionsWidth[1] = 800;
	resolutionsWidth[2] = 1024;
	resolutionsWidth[3] = 1280;
	resolutionsWidth[4] = 1280;
	resolutionsWidth[5] = 1280;
	resolutionsWidth[6] = 1600;
	resolutionsWidth[7] = 1680;
	
	resolutionsHeight[0] = 480;
	resolutionsHeight[1] = 600;
	resolutionsHeight[2] = 768;
	resolutionsHeight[3] = 800;
	resolutionsHeight[4] = 960;
	resolutionsHeight[5] = 1024;
	resolutionsHeight[6] = 1200;
	resolutionsHeight[7] = 1050;

	currentResolution	= 0;
	fullscreen			= false;
	multisampling		= 0;

	load();
}

/**
 * Destructor
 */
MenuOptions::~MenuOptions()
{

}

/**
 * Load the informatuions
 */
void MenuOptions::load()
{
	int  w			= op->getScreenWidth	(true);
	int  h			= op->getScreenHeight	(true);
	fullscreen		= op->isFullScreen		(true);
	multisampling	= op->getMultisampling	(true);

	for( int i=0; i<NBRESULTIONS; i++ )
		if( resolutionsWidth[i] == w && resolutionsHeight[i] == h )
			currentResolution = i;
}

void MenuOptions::refresh( long time )
{
	//IF PRESS B1
	if( ctrl->isB1(1) || ctrl->isB1(2) || ctrl->isEnter() )
	{
		ctrl->initB1(1);
		ctrl->initB1(2);
		ctrl->initEnter();

		switch( state )
		{
			case 1:
				currentResolution++;

				if( currentResolution>=NBRESULTIONS)
					currentResolution = 0;
				break;

			case 2:
				fullscreen = !fullscreen;
				break;
			case 3:

				// CHANGE THE MULTISAMPLING
				if( multisampling > 0 && multisampling < 16 )
					multisampling *= 2;
				else if( multisampling == 0 )
					multisampling  = 2;

				if( ! DirectXManager::getInstance().testMultisampling( multisampling ) )
					multisampling = 0;

				break;
			default:
				op->save( resolutionsWidth[currentResolution], resolutionsHeight[currentResolution], fullscreen, multisampling );
				action = state;
		}

		beep2->play();
	}

	//IF PRESS B2
	if( ctrl->isB2(1) || ctrl->isB2(2) || ctrl->isEnter()  )
	{
		ctrl->initB2(1);
		ctrl->initB2(2);
		ctrl->initEnter();

		switch( state )
		{
			case 1:
				currentResolution--;

				if( currentResolution<0)
					currentResolution = NBRESULTIONS -1 ;
				break;

			case 2:
				fullscreen = !fullscreen;
				break;

			case 3:

				// CHANGE THE MULTISAMPLING
				if( multisampling > 2 )
					multisampling /= 2;
				else
					multisampling = 0;

				if( ! DirectXManager::getInstance().testMultisampling( multisampling ) )
					multisampling = 0;
		}

		beep2->play();
	}

	if( timeLastKey < MENU_TIME_BETWEEN_TWO_KEYS )
	{
		timeLastKey += time;

		if( ! ctrl->isUp(1) && ! ctrl->isDown(1) && ! ctrl->isLeft(1) && ! ctrl->isRight(1) && ! ctrl->isUp(2) && ! ctrl->isDown(2) && ! ctrl->isLeft(2) && ! ctrl->isRight(2) )
			timeLastKey = MENU_TIME_BETWEEN_TWO_KEYS;
	}
	else
	{
		if( ctrl->isUp(1) || ctrl->isUp(2) )
		{
			state --;

			if( state <= 0 )
				state = nbElements;

			timeLastKey = 0;

			beep->play();
		}
		else if( ctrl->isDown(1) || ctrl->isDown(2) )
		{
			state ++;

			if( state > nbElements )
				state = 1;

			timeLastKey = 0;

			beep->play();
		}
	}
}

void MenuOptions::draw()
{
	Menu::draw();

	// RESOLUTION

	std::ostringstream res;
	res << resolutionsWidth[currentResolution] << " x " << resolutionsHeight[currentResolution];

	text->drawText(	font	,
		res.str(), 
		(int)(op->getScreenWidth() /2 - op->getScreenWidth() /3.5) ,
		(int)(op->getScreenHeight()/3 ) , 
		(int)(op->getScreenWidth() /2 + op->getScreenWidth() /3.5) ,
		(int)(op->getScreenHeight()/3 + op->getScreenHeight()/MENU_SIZE_FONT) ,
		255,255,255,255, 
		DT_RIGHT | DT_VCENTER);

	std::string fs;

	// FULL SCREEN

	if( fullscreen )
		fs = "yes";
	else
		fs = "no";

	text->drawText(	font	,
		 fs, 
		(int)(op->getScreenWidth() /2 - op->getScreenWidth() /3.5) ,
		(int)(op->getScreenHeight()/3 + op->getScreenHeight()/MENU_SIZE_FONT*1.5) , 
		(int)(op->getScreenWidth() /2 + op->getScreenWidth() /3.5) ,
		(int)(op->getScreenHeight()/3 + op->getScreenHeight()/MENU_SIZE_FONT*1.5 + op->getScreenHeight()/MENU_SIZE_FONT) ,
		255,255,255,255, 
		DT_RIGHT | DT_VCENTER);



	std::ostringstream mul;
	mul << multisampling << "x";

	text->drawText(	font	,
		 mul.str(), 
		(int)(op->getScreenWidth() /2 - op->getScreenWidth() /3.5) ,
		(int)(op->getScreenHeight()/3 + op->getScreenHeight()/MENU_SIZE_FONT*3) , 
		(int)(op->getScreenWidth() /2 + op->getScreenWidth() /3.5) ,
		(int)(op->getScreenHeight()/3 + op->getScreenHeight()/MENU_SIZE_FONT*3 + op->getScreenHeight()/MENU_SIZE_FONT) ,
		255,255,255,255, 
		DT_RIGHT | DT_VCENTER);
}

#endif
