// WIN INCLUDES
#define DIRECTINPUT_VERSION 0x0800
#include <windows.h>
#include <windowsx.h>
#include <dinput.h>
#include <d3d9.h>
#include <d3dx9.h>


// MY INCLUDES
#include "Managers/SpritesManager.h"
#include "Managers/MeshesManager.h"
#include "Managers/TextManager.h"
#include "Managers/DirectXManager.h"
#include "Managers/ControlsManager.h"
#include "Managers/GameManager.h"
#include "Managers/OptionManager.h"
#include "Managers/SoundsManager.h"
#include "Managers/LightManager.h"


// LIBS
#pragma comment (lib, "dinput8.lib")
#pragma comment (lib, "dxguid.lib")
#pragma comment (lib, "d3d9.lib")
#pragma comment (lib, "d3dx9.lib")
#pragma comment (lib, "OpenAL32.lib")
#pragma comment (lib, "libsndfile-1.lib")


// MANAGERS
ControlsManager * ctrl;
SpritesManager  * sp;
MeshesManager   * mesh;
TextManager     * text;
DirectXManager  * dx;
GameManager     * game;
OptionManager   * op;
SoundsManager   * snd;
LightManager	* lght;

// PROTOTYPES
void initD3D(HWND hWnd);
void refresh( long temps );
void draw();
void cleanD3D();
LRESULT CALLBACK WindowProc(HWND hWnd, UINT message, WPARAM wParam,LPARAM lParam);
/**
 * Win Main
 */
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine,int nCmdShow)
{
	// MANAGERS
	dx   = &DirectXManager	::getInstance();
	ctrl = &ControlsManager	::getInstance();
	sp   = &SpritesManager	::getInstance();
	mesh = &MeshesManager	::getInstance();
	text = &TextManager		::getInstance();
	game = &GameManager		::getInstance();
	op   = &OptionManager	::getInstance();
	snd	 = &SoundsManager   ::getInstance();
	lght = &LightManager	::getInstance();
	
	// WIN
    HWND		hWnd;
    WNDCLASSEX	wc;

    // CLEAR THE STRUCT
    ZeroMemory(&wc, sizeof(WNDCLASSEX));

    // FILL THE STRUCT
    wc.cbSize			= sizeof(WNDCLASSEX);
    wc.style			= CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc		= (WNDPROC) WindowProc;
    wc.hInstance		= hInstance;
    wc.hCursor			= LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground	= (HBRUSH) COLOR_WINDOW;
    wc.lpszClassName	= "RobotFrisbee";

    // REGISTER THE WIN
    RegisterClassEx(&wc);

	// IF THE WINDOW IS FULL SCREEN .. OR NOT
	if( op->isFullScreen() )
		hWnd = CreateWindowEx(NULL,"RobotFrisbee","RobotFrisbee",WS_EX_TOPMOST | WS_POPUP, 0,  0, op->getScreenWidth(), op->getScreenHeight(), NULL, NULL, hInstance,NULL);	
	else
		hWnd = CreateWindowEx(NULL,"RobotFrisbee","RobotFrisbee",WS_OVERLAPPED | WS_SYSMENU | WS_MINIMIZEBOX | WS_VISIBLE,100,100,op->getScreenWidth(),op->getScreenHeight(), NULL,NULL, hInstance, NULL);	

	// SHOW THE WINDOW
    ShowWindow(hWnd, nCmdShow);

    //INITS
	dx  ->init(hInstance, hWnd);
    ctrl->init();
	sp  ->init();
	mesh->init();
	text->init();
	snd ->init();
	game->init();
	lght->init();

	// Creation Light
	{
		D3DCOLORVALUE d = {1.0f,1.0f,1.0f,1.0f};
		D3DCOLORVALUE s = {0.0f,0.0f,0.0f,1.0f};
		D3DCOLORVALUE a = {0.0f,0.0f,0.0f,1.0f};
		D3DVECTOR p		= {0.0f,0.0f,10.0f};
		lght->addLightPoint(d,s,a,p,50.0f,1.0f,0.0f,0.0f);
	} // Destruction d, s, a and p.

	// CALACULATE THE TIME BETWEEN TWO FRAMES
	long time		  = GetTickCount();
	long previousTime = GetTickCount();

    MSG msg;
    while(TRUE)
    {
		// CALACULATE THE TIME BETWEEN TWO FRAMES
		time = GetTickCount() - previousTime;
		previousTime = GetTickCount();

		//IF THE FRAMERATE IS UNDER 30FPS SLOW THE GAME
		if( time > 1000/30 )
			time = 1000/30;

        if (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
        {
            if (msg.message == WM_QUIT)
                break;

            TranslateMessage(&msg);
            DispatchMessage(&msg);
        } 

		// KEYS + REFRESH + DISPLAY
		refresh(time);
		draw();

		Sleep( 10 );
    }

    return (int)msg.wParam;
}


/**
 * Handler
 */
LRESULT CALLBACK WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch(message)
    {
        case WM_DESTROY:
		{
			PostQuitMessage(0);
			return 0;
		} 
    }
    return DefWindowProc (hWnd, message, wParam, lParam);
}


/**
 * Draw the frame
 */
void refresh( long time )
{
	ctrl->refresh();
	game->refresh(time);
}


/**
 * Draw the frame
 */
void draw()
{
	// CLEAR
    dx->d3ddev->Clear(0, NULL, D3DCLEAR_TARGET , D3DCOLOR_XRGB(80, 80, 80), 1.0f, 0);
    dx->d3ddev->Clear(0, NULL, D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB(0, 0, 0), 1.0f, 0);

	// BEGIN
    dx->d3ddev->BeginScene();

    // SET UP THE TRANSFORMS
    D3DXMATRIX matView;
    D3DXMatrixLookAtLH(&matView,
    &D3DXVECTOR3 (0.0f, 0.0f, 8.0f),	// CAM
    &D3DXVECTOR3 (0.0f, 0.0f, 0.0f),	// LOOK-AT
    &D3DXVECTOR3 (0.0f, 1.0f, 0.0f));	// UP
    dx->d3ddev->SetTransform(D3DTS_VIEW, &matView);


	// PROJECTION
    D3DXMATRIX matProjection;	
	D3DXMatrixPerspectiveFovLH(&matProjection,  D3DXToRadian(45), op->getScreenWidth() / (float)op->getScreenHeight() ,1.0f, 100.0f);
    dx->d3ddev->SetTransform(D3DTS_PROJECTION, &matProjection);

	// DRAW THE GAME
	game->draw();

	// END
    dx->d3ddev->EndScene(); 
    dx->d3ddev->Present(NULL, NULL, NULL, NULL);
}
